@@ -209,6 +209,18 @@ static VALUE rb_gsl_cdf_lognormal_Qinv(VALUE obj, VALUE x, VALUE a, VALUE b)
   return rb_gsl_eval_pdf_cdf3(x, a, b,  gsl_cdf_lognormal_Qinv);
 }  
 
+#ifdef GSL_1_6_LATER
+static VALUE rb_gsl_cdf_exppow_P(VALUE obj, VALUE x, VALUE a, VALUE b)
+{
+  return rb_gsl_eval_pdf_cdf3(x, a, b, gsl_cdf_exppow_P);
+}  
+
+static VALUE rb_gsl_cdf_exppow_Q(VALUE obj, VALUE x, VALUE a, VALUE b)
+{
+  return rb_gsl_eval_pdf_cdf3(x, a, b, gsl_cdf_exppow_Q);
+}  
+#endif
+
 static VALUE rb_gsl_cdf_chisq_P(VALUE obj, VALUE x, VALUE mu)
 {
   return rb_gsl_eval_pdf_cdf2(x, mu, gsl_cdf_chisq_P);
@@ -548,5 +560,14 @@ void Init_gsl_cdf(VALUE module)
   rb_define_module_function(mgsl_cdf, "gumbel2_Pinv", rb_gsl_cdf_gumbel2_Pinv, 3);
   rb_define_module_function(mgsl_cdf, "gumbel2_Qinv", rb_gsl_cdf_gumbel2_Qinv, 3);
 
+
+#ifdef GSL_1_6_LATER
+  rb_define_module_function(module, "cdf_exppow_P", rb_gsl_cdf_exppow_P, 3);
+  rb_define_module_function(module, "cdf_exppow_Q", rb_gsl_cdf_exppow_Q, 3);
+
+  rb_define_module_function(mgsl_cdf, "exppow_P", rb_gsl_cdf_exppow_P, 3);
+  rb_define_module_function(mgsl_cdf, "exppow_Q", rb_gsl_cdf_exppow_Q, 3);
+#endif
+
 }
 #endif
