@@ -10,26 +10,21 @@
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
 #include "rb_gsl_config.h"
-
 #include "rb_gsl_complex.h"
 #include "rb_gsl_array.h"
 
 VALUE cgsl_complex;
 
-void rb_gsl_complex_free(gsl_complex *c);
 static VALUE rb_gsl_complex_new(int argc, VALUE *argv, VALUE klass)
 {
   gsl_complex *c = NULL;
-  VALUE obj, re, im;
-  obj = Data_Make_Struct(klass, gsl_complex, 0, rb_gsl_complex_free, c); 
+  VALUE obj;
+  obj = Data_Make_Struct(klass, gsl_complex, 0, free, c); 
   switch (argc) {
   case 1:
     switch (TYPE(argv[0])) {
     case T_ARRAY:
-      re = rb_ary_entry(argv[0], 0);
-      im = rb_ary_entry(argv[0], 1);
-      Need_Float(re); Need_Float(im);
-      *c = gsl_complex_rect(NUM2DBL(re), NUM2DBL(im));
+      *c = ary2complex(argv[0]);
       break;
     case T_FLOAT:
     case T_FIXNUM:
@@ -52,11 +47,6 @@ static VALUE rb_gsl_complex_new(int argc, VALUE *argv, VALUE klass)
   return obj;
 }
 
-void rb_gsl_complex_free(gsl_complex *c)
-{
-  free((gsl_complex *) c);
-}
-
 gsl_complex* make_complex(double re, double im)
 {
   gsl_complex *c = NULL;
@@ -70,7 +60,7 @@ static VALUE rb_gsl_complex_polar(VALUE klass, VALUE r, VALUE theta)
   VALUE obj;
   gsl_complex *c = NULL;
   Need_Float(r); Need_Float(theta);
-  obj = Data_Make_Struct(klass, gsl_complex, 0, rb_gsl_complex_free, c); 
+  obj = Data_Make_Struct(klass, gsl_complex, 0, free, c); 
   *c = gsl_complex_polar(NUM2DBL(r), NUM2DBL(theta));
   return obj;
 }
@@ -113,16 +103,12 @@ static VALUE rb_gsl_complex_imag(VALUE obj)
 static VALUE rb_gsl_complex_set(int argc, VALUE *argv, VALUE obj)
 {
   gsl_complex *c = NULL;
-  VALUE re, im;
   Data_Get_Struct(obj, gsl_complex, c);
   switch (argc) {
   case 1:
     switch (TYPE(argv[0])) {
     case T_ARRAY:
-      re = rb_ary_entry(argv[0], 0);
-      im = rb_ary_entry(argv[0], 1);
-      Need_Float(re);      Need_Float(im);
-      *c = gsl_complex_rect(NUM2DBL(re), NUM2DBL(im));
+      *c = ary2complex(argv[0]);
       break;
     case T_FLOAT:
     case T_BIGNUM:
@@ -131,6 +117,7 @@ static VALUE rb_gsl_complex_set(int argc, VALUE *argv, VALUE obj)
       break;
     default:
       rb_raise(rb_eTypeError, "Array or Numeric expected");
+      break;
     }
     break;
   case 2:
@@ -166,7 +153,7 @@ static VALUE rb_gsl_complex_print(VALUE obj)
 {
   gsl_complex *c = NULL;
   Data_Get_Struct(obj, gsl_complex, c);
-  fprintf(stdout, "[%g %g] \n", GSL_REAL(*c), GSL_IMAG(*c));
+  fprintf(stdout, "[%4.3e %4.3e] \n", GSL_REAL(*c), GSL_IMAG(*c));
   return obj;
 }
 
@@ -182,14 +169,6 @@ static VALUE rb_gsl_complex_printf(VALUE obj, VALUE s)
   return obj;
 }
 
-static VALUE rb_gsl_complex_inspect(VALUE obj)
-{
-  gsl_complex *c = NULL;
-  Data_Get_Struct(obj, gsl_complex, c);
-  fprintf(stdout, "[%g %g]\n", GSL_REAL(*c), GSL_IMAG(*c));
-  return obj;
-}
-
 static VALUE rb_gsl_complex_return_double(double (*func)(gsl_complex), VALUE obj);
 static VALUE rb_gsl_complex_return_double(double (*func)(gsl_complex), VALUE obj)
 {
@@ -231,7 +210,7 @@ static VALUE rb_gsl_complex_arithmetics2(gsl_complex (*func)(gsl_complex, double
   Data_Get_Struct(obj, gsl_complex, a);
   x = NUM2DBL(xx);
   tmp = (*func)(*a, x);
-  obj2 = Data_Make_Struct(cgsl_complex, gsl_complex, 0, rb_gsl_complex_free, c); 
+  obj2 = Data_Make_Struct(cgsl_complex, gsl_complex, 0, free, c); 
   *c = tmp;
   return obj2;
 }
@@ -281,31 +260,73 @@ static VALUE rb_gsl_complex_operate2(gsl_complex (*func)(gsl_complex), int argc,
 
 static VALUE rb_gsl_complex_operate(gsl_complex (*func)(gsl_complex), VALUE obj)
 {
-  gsl_complex *c = NULL, *cnew = NULL, tmp;
+  gsl_complex *c = NULL, *cnew = NULL;
   Data_Get_Struct(obj, gsl_complex, c);
-  tmp = (*func)(*c);
   cnew = ALLOC(gsl_complex);
-  *cnew = tmp;
-  return Data_Wrap_Struct(cgsl_complex, 0, rb_gsl_complex_free, cnew); 
+  *cnew = (*func)(*c);
+  return Data_Wrap_Struct(cgsl_complex, 0, free, cnew); 
 }
 
 static VALUE rb_gsl_complex_operate2(gsl_complex (*func)(gsl_complex), int argc, VALUE *argv, VALUE obj)
 {
   gsl_complex *c = NULL, *cnew = NULL, tmp;
+  gsl_vector_complex *v, *vnew;
+  gsl_matrix_complex *m, *mnew;
+  size_t i, j;
   switch (TYPE(obj)) {
   case T_MODULE:
   case T_CLASS:
   case T_OBJECT:
-    CHECK_COMPLEX(argv[0]);
-    Data_Get_Struct(argv[0], gsl_complex, c);
+    switch (argc) {
+    case 1:
+      switch (TYPE(argv[0])) {
+      case T_ARRAY:
+	tmp = ary2complex(argv[0]);
+	c = &tmp;
+	break;
+      default:
+	if (VECTOR_COMPLEX_P(argv[0])) {
+	  Data_Get_Struct(argv[0], gsl_vector_complex, v);
+	  vnew = gsl_vector_complex_alloc(v->size);
+	  for (i = 0; i < v->size; i++) {
+	    c = GSL_COMPLEX_AT(v, i);
+	    gsl_vector_complex_set(vnew, i, (*func)(*c));
+	  }
+	  return Data_Wrap_Struct(cgsl_vector_complex, 0, gsl_vector_free, vnew);
+	} else if (MATRIX_COMPLEX_P(obj)) {
+	  Data_Get_Struct(obj, gsl_matrix_complex, m);
+	  mnew = gsl_matrix_complex_alloc(m->size1, m->size2);
+	  for (i = 0; i < m->size1; i++) {
+	    for (j = 0; j < m->size2; j++) {
+	      gsl_matrix_complex_set(mnew, i, j, 
+				     (*func)(gsl_matrix_complex_get(m, i, j)));
+	    }
+	  }
+	  return Data_Wrap_Struct(cgsl_matrix_complex, 0, gsl_matrix_complex_free, mnew);
+	} else {
+	  CHECK_COMPLEX(argv[0]);
+	  Data_Get_Struct(argv[0], gsl_complex, c);
+	}
+	break;
+      }
+      break;
+    case 2:
+      c = &tmp;
+      GSL_SET_REAL(c, NUM2DBL(argv[0]));
+      GSL_SET_IMAG(c, NUM2DBL(argv[1]));
+      break;
+    default:
+      rb_raise(rb_eArgError, "wrong number of arguments (%d for 1 or 2)", argc);
+      break;
+    }
     break;
   default:
     Data_Get_Struct(obj, gsl_complex, c);
+    break;
   }
-  tmp = (*func)(*c);
   cnew = ALLOC(gsl_complex);
-  *cnew = tmp;
-  return Data_Wrap_Struct(cgsl_complex, 0, rb_gsl_complex_free, cnew); 
+  *cnew = (*func)(*c);
+  return Data_Wrap_Struct(cgsl_complex, 0, free, cnew); 
 }
 
 static VALUE rb_gsl_complex_conjugate(VALUE obj)
@@ -323,11 +344,7 @@ static VALUE rb_gsl_complex_negative(VALUE obj)
   return rb_gsl_complex_operate(gsl_complex_negative, obj);
 }
 
-static VALUE rb_gsl_complex_sqrt(int argc, VALUE *argv, VALUE obj)
-{
-  return rb_gsl_complex_operate2(gsl_complex_sqrt, argc, argv, obj);
-}
-
+/* singleton */
 static VALUE rb_gsl_complex_sqrt_real(VALUE obj, VALUE x)
 {
   gsl_complex *cnew = NULL, tmp;
@@ -335,62 +352,198 @@ static VALUE rb_gsl_complex_sqrt_real(VALUE obj, VALUE x)
   tmp = gsl_complex_sqrt_real(NUM2DBL(x));
   cnew = ALLOC(gsl_complex);
   *cnew = tmp;
-  return Data_Wrap_Struct(cgsl_complex, 0, rb_gsl_complex_free, cnew); 
+  return Data_Wrap_Struct(cgsl_complex, 0, free, cnew); 
 }
 
-static VALUE rb_gsl_complex_pow(int argc, VALUE *argv, VALUE obj)
+static VALUE rb_gsl_complex_sqrt(int argc, VALUE *argv, VALUE obj)
 {
-  gsl_complex *c = NULL, *a = NULL, *cnew = NULL, tmp;
+  gsl_complex *z, tmp;
   switch (TYPE(obj)) {
-  case T_MODULE:
-  case T_CLASS:
-  case T_OBJECT:
-    if (argc != 2) rb_raise(rb_eArgError, "wrong number of arguments (%d for 2)", argc);
-    CHECK_COMPLEX(argv[0]);
-    CHECK_COMPLEX(argv[1]);
-    Data_Get_Struct(argv[0], gsl_complex, c);
-    Data_Get_Struct(argv[1], gsl_complex, a);
+  case T_MODULE: case T_CLASS: case T_OBJECT:
+    switch (argc) {
+    case 1:
+      switch (TYPE(argv[0])) {
+      case T_FIXNUM: case T_FLOAT:
+	return rb_gsl_complex_sqrt_real(obj, argv[0]);
+	break;
+      case T_ARRAY:
+	tmp = ary2complex(argv[0]);
+	z = ALLOC(gsl_complex);
+	*z = gsl_complex_sqrt(tmp);
+	return Data_Wrap_Struct(cgsl_complex, 0, free, z);
+	break;
+      default:
+	CHECK_COMPLEX(argv[0]);
+	Data_Get_Struct(argv[0], gsl_complex, z);
+	tmp = *z;
+	z = ALLOC(gsl_complex);
+	*z = gsl_complex_sqrt(tmp);
+	return Data_Wrap_Struct(cgsl_complex, 0, free, z);      
+	break;
+      }
+      break;
+    case 2:
+      z = ALLOC(gsl_complex);
+      GSL_SET_REAL(&tmp, NUM2DBL(argv[0]));
+      GSL_SET_IMAG(&tmp, NUM2DBL(argv[1]));
+      *z = gsl_complex_sqrt(tmp);
+      return Data_Wrap_Struct(cgsl_complex, 0, free, z);     
+      break;
+    default:
+      rb_raise(rb_eArgError, "wrong number of arguments (%d for 1 or 2)", argc);
+      break;
+    }
     break;
   default:
-    if (argc != 1) rb_raise(rb_eArgError, "wrong number of arguments (%d for 1)", argc);
-    CHECK_COMPLEX(obj);
-    CHECK_COMPLEX(argv[0]);
-    Data_Get_Struct(obj, gsl_complex, c);
-    Data_Get_Struct(argv[0], gsl_complex, a);
+    return rb_gsl_complex_operate2(gsl_complex_sqrt, argc, argv, obj);
     break;
   }
-  tmp = gsl_complex_pow(*c, *a);
-  cnew = ALLOC(gsl_complex);
-  *cnew = tmp;
-  return Data_Wrap_Struct(cgsl_complex, 0, rb_gsl_complex_free, cnew); 
 }
 
-static VALUE rb_gsl_complex_pow_real(int argc, VALUE *argv, VALUE obj)
+VALUE rb_gsl_complex_pow(int argc, VALUE *argv, VALUE obj)
 {
-  double a;
-  gsl_complex *c = NULL, *cnew = NULL, tmp;
+  gsl_complex *c = NULL, *a = NULL, *cnew = NULL, tmpc, tmpa;
+  gsl_vector_complex *v, *vnew;
+  gsl_matrix_complex *m, *mnew;
+  size_t i, j;
   switch (TYPE(obj)) {
   case T_MODULE:
   case T_CLASS:
   case T_OBJECT:
-    if (argc != 2) rb_raise(rb_eArgError, "wrong number of arguments (%d for 2)", argc);
-    CHECK_COMPLEX(argv[0]);
+    if (argc != 2) 
+      rb_raise(rb_eArgError, "wrong number of arguments (%d for 2)", argc);
+    switch (TYPE(argv[1])) {
+    case T_ARRAY:
+      tmpa = ary2complex(argv[1]);
+      a = &tmpa;
+      break;
+    case T_FLOAT: case T_FIXNUM: 
+      return rb_gsl_complex_pow_real(argc, argv, obj);
+      break;
+    default:
+      CHECK_COMPLEX(argv[1]);
+      Data_Get_Struct(argv[1], gsl_complex, a);
+      break;
+    }
+
+    switch (TYPE(argv[0])) {
+    case T_ARRAY:
+      tmpc = ary2complex(argv[0]);
+      c = &tmpc;
+      break;
+    default:
+      if (VECTOR_COMPLEX_P(argv[0])) {
+	Data_Get_Struct(argv[0], gsl_vector_complex, v);
+	vnew = gsl_vector_complex_alloc(v->size);
+	for (i = 0; i < v->size; i++) {
+	  c = GSL_COMPLEX_AT(v, i);
+	  gsl_vector_complex_set(vnew, i, gsl_complex_pow(*c, *a));
+	}
+	return Data_Wrap_Struct(cgsl_vector_complex, 0, gsl_vector_complex_free, vnew);
+      }
+      if (MATRIX_COMPLEX_P(argv[0])) {
+	Data_Get_Struct(argv[0], gsl_matrix_complex, m);
+	mnew = gsl_matrix_complex_alloc(m->size1, m->size2);
+	for (i = 0; i < m->size1; i++) {
+	  for (j = 0; j < m->size2; j++) {
+	    c = gsl_matrix_complex_ptr(m, i, j);
+	    gsl_matrix_complex_set(mnew, i, j, gsl_complex_pow(*c, *a));
+	  }
+	}
+	return Data_Wrap_Struct(cgsl_matrix_complex, 0, gsl_matrix_complex_free, mnew);
+      }
+      CHECK_COMPLEX(argv[0]);
+      Data_Get_Struct(argv[0], gsl_complex, c);
+      break;
+    }
+
+    break;
+  default:
+    if (argc != 1) 
+      rb_raise(rb_eArgError, "wrong number of arguments (%d for 1)", argc);
+    CHECK_COMPLEX(obj);
+    Data_Get_Struct(obj, gsl_complex, c);
+    switch (TYPE(argv[0])) {
+    case T_ARRAY:
+      tmpa = ary2complex(argv[0]);
+      a = &tmpa;
+      break;
+    case T_FLOAT: case T_FIXNUM: 
+      return rb_gsl_complex_pow_real(argc, argv, obj);
+      break;
+    default:
+      CHECK_COMPLEX(argv[0]);
+      Data_Get_Struct(argv[0], gsl_complex, a);
+      break;
+    }
+    break;
+  }
+  cnew = ALLOC(gsl_complex);
+  *cnew = gsl_complex_pow(*c, *a);
+  return Data_Wrap_Struct(cgsl_complex, 0, free, cnew); 
+}
+
+VALUE rb_gsl_complex_pow_real(int argc, VALUE *argv, VALUE obj)
+{
+  double a = 1;
+  gsl_complex *c = NULL, *cnew = NULL, tmpc;
+  gsl_vector_complex *v, *vnew;
+  gsl_matrix_complex *m, *mnew;
+  size_t i, j;
+  switch (TYPE(obj)) {
+  case T_MODULE:
+  case T_CLASS:
+  case T_OBJECT:
+    if (argc != 2) 
+      rb_raise(rb_eArgError, "wrong number of arguments (%d for 2)", argc);
+    switch (TYPE(argv[0])) {
+    case T_ARRAY:
+      tmpc = ary2complex(argv[0]);
+      c = &tmpc;
+      a = NUM2DBL(argv[1]);
+      break;
+    default:
+      if (VECTOR_COMPLEX_P(argv[0])) {
+	Data_Get_Struct(argv[0], gsl_vector_complex, v);
+	vnew = gsl_vector_complex_alloc(v->size);
+	a = NUM2DBL(argv[1]);
+	for (i = 0; i < v->size; i++) {
+	  c = GSL_COMPLEX_AT(v, i);
+	  tmpc = gsl_complex_pow_real(*c, a);
+	  gsl_vector_complex_set(vnew, i, tmpc);
+	}
+	return Data_Wrap_Struct(cgsl_vector_complex, 0, gsl_vector_free, vnew);
+      }
+      if (MATRIX_COMPLEX_P(argv[0])) {
+	Data_Get_Struct(argv[0], gsl_matrix_complex, m);
+	mnew = gsl_matrix_complex_alloc(m->size1, m->size2);
+	for (i = 0; i < m->size1; i++) {
+	  for (j = 0; j < m->size2; j++) {
+	    tmpc = gsl_complex_pow_real(gsl_matrix_complex_get(m, i, j), a);
+	    gsl_matrix_complex_set(mnew, i, j, tmpc);
+	  }
+	}
+	return Data_Wrap_Struct(cgsl_matrix_complex, 0, gsl_matrix_complex_free, mnew);
+      }
+      CHECK_COMPLEX(argv[0]);
+      Data_Get_Struct(argv[0], gsl_complex, c);
+      break;
+    }
     Need_Float(argv[1]);
-    Data_Get_Struct(argv[0], gsl_complex, c);
     a = NUM2DBL(argv[1]);
     break;
   default:
-    if (argc != 1) rb_raise(rb_eArgError, "wrong number of arguments (%d for 1)", argc);
+    if (argc != 1) 
+      rb_raise(rb_eArgError, "wrong number of arguments (%d for 1)", argc);
     CHECK_COMPLEX(obj);
     Need_Float(argv[0]);
     Data_Get_Struct(obj, gsl_complex, c);
     a = NUM2DBL(argv[0]);
     break;
   }
-  tmp = gsl_complex_pow_real(*c, a);
   cnew = ALLOC(gsl_complex);
-  *cnew = tmp;
-  return Data_Wrap_Struct(cgsl_complex, 0, rb_gsl_complex_free, cnew); 
+  *cnew = gsl_complex_pow_real(*c, a);
+  return Data_Wrap_Struct(cgsl_complex, 0, free, cnew); 
 }
 
 static VALUE rb_gsl_complex_exp(int argc, VALUE *argv, VALUE obj)
@@ -410,29 +563,56 @@ static VALUE rb_gsl_complex_log10(int argc, VALUE *argv, VALUE obj)
 
 static VALUE rb_gsl_complex_log_b(int argc, VALUE *argv, VALUE obj)
 {
-  gsl_complex *z = NULL, *b = NULL, *cnew = NULL, tmp;
+  gsl_complex *c = NULL, *a = NULL, *cnew = NULL, tmpc, tmpa;
   switch (TYPE(obj)) {
   case T_MODULE:
   case T_CLASS:
   case T_OBJECT:
-    if (argc != 2) rb_raise(rb_eArgError, "wrong number of arguments (%d for 2)", argc);
-    CHECK_COMPLEX(argv[0]);
-    CHECK_COMPLEX(argv[1]);
-    Data_Get_Struct(argv[0], gsl_complex, z);
-    Data_Get_Struct(argv[1], gsl_complex, b);
+    if (argc != 2) 
+      rb_raise(rb_eArgError, "wrong number of arguments (%d for 2)", argc);
+    switch (TYPE(argv[1])) {
+    case T_ARRAY:
+      tmpa = ary2complex(argv[1]);
+      a = &tmpa;
+      break;
+    default:
+      CHECK_COMPLEX(argv[1]);
+      Data_Get_Struct(argv[1], gsl_complex, a);
+      break;
+    }
+
+    switch (TYPE(argv[0])) {
+    case T_ARRAY:
+      tmpc = ary2complex(argv[0]);
+      c = &tmpc;
+      break;
+    default:
+      CHECK_COMPLEX(argv[0]);
+      Data_Get_Struct(argv[0], gsl_complex, c);
+      break;
+    }
+
     break;
   default:
-    if (argc != 1) rb_raise(rb_eArgError, "wrong number of arguments (%d for 1)", argc);
+    if (argc != 1) 
+      rb_raise(rb_eArgError, "wrong number of arguments (%d for 1)", argc);
     CHECK_COMPLEX(obj);
-    CHECK_COMPLEX(argv[0]);
-    Data_Get_Struct(obj, gsl_complex, z);
-    Data_Get_Struct(argv[0], gsl_complex, b);
+    Data_Get_Struct(obj, gsl_complex, c);
+    switch (TYPE(argv[0])) {
+    case T_ARRAY:
+      tmpa = ary2complex(argv[0]);
+      a = &tmpa;
+      break;
+    default:
+      CHECK_COMPLEX(argv[0]);
+      Data_Get_Struct(argv[0], gsl_complex, a);
+      break;
+    }
     break;
   }
-  tmp = gsl_complex_log_b(*z, *b);
   cnew = ALLOC(gsl_complex);
-  *cnew = tmp;
-  return Data_Wrap_Struct(cgsl_complex, 0, rb_gsl_complex_free, cnew); 
+  *cnew = gsl_complex_log_b(*c, *a);
+  return Data_Wrap_Struct(cgsl_complex, 0, free, cnew); 
 }
 
 static VALUE rb_gsl_complex_sin(int argc, VALUE *argv, VALUE obj)
@@ -480,7 +660,7 @@ static VALUE rb_gsl_complex_arcsin_real(VALUE obj, VALUE xx)
   tmp = gsl_complex_arcsin_real(x);
   cnew = ALLOC(gsl_complex);
   *cnew = tmp;
-  return Data_Wrap_Struct(cgsl_complex, 0, rb_gsl_complex_free, cnew); 
+  return Data_Wrap_Struct(cgsl_complex, 0, free, cnew); 
 }
 
 static VALUE rb_gsl_complex_arccos(int argc, VALUE *argv, VALUE obj)
@@ -497,7 +677,7 @@ static VALUE rb_gsl_complex_arccos_real(VALUE obj, VALUE xx)
   tmp = gsl_complex_arccos_real(x);
   cnew = ALLOC(gsl_complex);
   *cnew = tmp;
-  return Data_Wrap_Struct(cgsl_complex, 0, rb_gsl_complex_free, cnew); 
+  return Data_Wrap_Struct(cgsl_complex, 0, free, cnew); 
 }
 
 static VALUE rb_gsl_complex_arctan(int argc, VALUE *argv, VALUE obj)
@@ -519,7 +699,7 @@ static VALUE rb_gsl_complex_arcsec_real(VALUE obj, VALUE xx)
   tmp = gsl_complex_arcsec_real(x);
   cnew = ALLOC(gsl_complex);
   *cnew = tmp;
-  return Data_Wrap_Struct(cgsl_complex, 0, rb_gsl_complex_free, cnew); 
+  return Data_Wrap_Struct(cgsl_complex, 0, free, cnew); 
 }
 
 static VALUE rb_gsl_complex_arccsc(int argc, VALUE *argv, VALUE obj)
@@ -536,7 +716,7 @@ static VALUE rb_gsl_complex_arccsc_real(VALUE obj, VALUE xx)
   tmp = gsl_complex_arccsc_real(x);
   cnew = ALLOC(gsl_complex);
   *cnew = tmp;
-  return Data_Wrap_Struct(cgsl_complex, 0, rb_gsl_complex_free, cnew); 
+  return Data_Wrap_Struct(cgsl_complex, 0, free, cnew); 
 }
 
 static VALUE rb_gsl_complex_arccot(int argc, VALUE *argv, VALUE obj)
@@ -588,7 +768,7 @@ static VALUE rb_gsl_complex_arccosh_real(VALUE obj, VALUE xx)
   tmp = gsl_complex_arccosh_real(x);
   cnew = ALLOC(gsl_complex);
   *cnew = tmp;
-  return Data_Wrap_Struct(cgsl_complex, 0, rb_gsl_complex_free, cnew); 
+  return Data_Wrap_Struct(cgsl_complex, 0, free, cnew); 
 }
 
 static VALUE rb_gsl_complex_arccosh(int argc, VALUE *argv, VALUE obj)
@@ -610,7 +790,7 @@ static VALUE rb_gsl_complex_arctanh_real(VALUE obj, VALUE xx)
   tmp = gsl_complex_arctanh_real(x);
   cnew = ALLOC(gsl_complex);
   *cnew = tmp;
-  return Data_Wrap_Struct(cgsl_complex, 0, rb_gsl_complex_free, cnew); 
+  return Data_Wrap_Struct(cgsl_complex, 0, free, cnew); 
 }
 
 static VALUE rb_gsl_complex_arcsech(int argc, VALUE *argv, VALUE obj)
@@ -660,6 +840,24 @@ static VALUE rb_gsl_complex_zero(VALUE obj)
   else return Qfalse;
 }
 
+static VALUE rb_gsl_complex_to_s(VALUE obj)
+{
+  char buf[256];
+  gsl_complex *z;
+  Data_Get_Struct(obj, gsl_complex, z);
+  sprintf(buf, "[ %f %f ]", GSL_REAL(*z), GSL_IMAG(*z));
+  return rb_str_new2(buf);
+}
+
+static VALUE rb_gsl_complex_inspect(VALUE obj)
+{
+  char buf[256];
+  VALUE str;
+  sprintf(buf, "%s\n", rb_class2name(CLASS_OF(obj)));
+  str = rb_str_new2(buf);
+  return rb_str_concat(str, rb_gsl_complex_to_s(obj));
+}
+
 void Init_gsl_complex(VALUE module)
 {
   cgsl_complex = rb_define_class_under(module, "Complex",  rb_cNumeric);
@@ -682,7 +880,6 @@ void Init_gsl_complex(VALUE module)
   rb_define_alias(cgsl_complex, "IMAG", "imag");
   rb_define_method(cgsl_complex, "print", rb_gsl_complex_print, 0);
   rb_define_method(cgsl_complex, "printf", rb_gsl_complex_printf, 1);
-  rb_define_method(cgsl_complex, "inspect", rb_gsl_complex_inspect, 0);
   rb_define_alias(cgsl_complex, "show", "print");
 
   rb_define_method(cgsl_complex, "set", rb_gsl_complex_set, -1);
@@ -698,7 +895,10 @@ void Init_gsl_complex(VALUE module)
   rb_define_alias(cgsl_complex, "SET_IMAG", "set_imag");
 
   rb_define_method(cgsl_complex, "arg", rb_gsl_complex_arg, 0);
+  rb_define_alias(cgsl_complex, "phase", "arg");
   rb_define_method(cgsl_complex, "abs", rb_gsl_complex_abs, 0);
+  rb_define_alias(cgsl_complex, "amp", "abs");
+  rb_define_alias(cgsl_complex, "mag", "abs");
   rb_define_method(cgsl_complex, "abs2", rb_gsl_complex_abs2, 0);
   rb_define_method(cgsl_complex, "logabs", rb_gsl_complex_logabs, 0);
 
@@ -797,4 +997,7 @@ void Init_gsl_complex(VALUE module)
   rb_define_method(cgsl_complex, "arcsech", rb_gsl_complex_arcsech, -1);
   rb_define_method(cgsl_complex, "arccsch", rb_gsl_complex_arccsch, -1);
   rb_define_method(cgsl_complex, "arccoth", rb_gsl_complex_arccoth, -1);
+
+  rb_define_method(cgsl_complex, "to_s", rb_gsl_complex_to_s, 0);
+  rb_define_method(cgsl_complex, "inspect", rb_gsl_complex_inspect, 0);
 }
