@@ -139,14 +139,13 @@ rescue
   raise("Check GSL>=0.9.4 is installed, and the command \"gsl-config\" is in search path.")
 end
 
-narray_config = dir_config("narray")
+#narray_config = dir_config("narray")
+narray_config = dir_config('narray',$sitearchdir,$sitearchdir)
 have_narray_h = have_header("narray.h")
 if narray_config
-#  if RUBY_PLATFORM =~ /cygwin|mingw/
-#    have_library("narray")
-#    CONFIG["DLDFLAGS"] << " -Wl,--out-implib=libnarray.a"
-#    $DLDFLAGS << " -Wl,--out-implib=libnarray.a"
-#  end
+  if RUBY_PLATFORM =~ /cygwin|mingw/
+    have_library("narray") || raise("ERROR: narray import library is not found")
+  end
 end
 
 have_func("round")
@@ -159,6 +158,14 @@ File.open("../lib/gsl.rb", "w") do |file|
   file.print("require\(\"gsl/oper.rb\"\)\n")
 end
 
+File.open("../lib/rbgsl.rb", "w") do |file|
+  if have_narray_h
+    file.print("require\(\"narray\"\)\n")
+  end
+  file.print("require\(\"rb_gsl\"\)\n")
+  file.print("require\(\"gsl/oper.rb\"\)\n")
+end
+
 srcs = %w(
 array 
 array_complex 
