@@ -21,6 +21,7 @@ VALUE cgsl_histogram_range;
 VALUE cgsl_histogram_bin;
 static VALUE cgsl_histogram_integ;
 
+static VALUE rb_gsl_histogram_alloc_from_file(VALUE klass, VALUE name);
 #ifdef GSL_0_9_4_LATER
 static VALUE rb_gsl_histogram_alloc(int argc, VALUE *argv, VALUE klass)
 {
@@ -41,6 +42,9 @@ static VALUE rb_gsl_histogram_alloc(int argc, VALUE *argv, VALUE klass)
       gsl_histogram_set_ranges(h, v->data, v->size);
       gsl_vector_free(v);
       break;
+    case T_STRING:
+      return rb_gsl_histogram_alloc_from_file(klass, argv[0]);
+      break;
     default:
       CHECK_VECTOR(argv[0]);
       Data_Get_Struct(argv[0], gsl_vector, v);
@@ -56,7 +60,7 @@ static VALUE rb_gsl_histogram_alloc(int argc, VALUE *argv, VALUE klass)
     n = FIX2INT(argv[0]);
     min = NUM2DBL(argv[1]);
     max = NUM2DBL(argv[2]);
-    h = gsl_histogram_alloc(n);
+    h = gsl_histogram_calloc(n);
     gsl_histogram_set_ranges_uniform(h, min, max);
     return Data_Wrap_Struct(klass, 0, gsl_histogram_free, h);
     break;
@@ -71,7 +75,7 @@ static VALUE rb_gsl_histogram_alloc(int argc, VALUE *argv, VALUE klass)
       n = FIX2INT(argv[0]);
       min = NUM2DBL(rb_ary_entry(argv[1], 0));
       max = NUM2DBL(rb_ary_entry(argv[1], 1));
-      h = gsl_histogram_alloc(n);
+      h = gsl_histogram_calloc(n);
       gsl_histogram_set_ranges_uniform(h, min, max);
       return Data_Wrap_Struct(klass, 0, gsl_histogram_free, h);
       break;
@@ -79,7 +83,7 @@ static VALUE rb_gsl_histogram_alloc(int argc, VALUE *argv, VALUE klass)
       CHECK_FIXNUM(argv[1]);
       v = make_cvector_from_rarray(argv[0]);
       size = FIX2INT(argv[1]);
-      h = gsl_histogram_alloc(size-1);
+      h = gsl_histogram_calloc(size-1);
       gsl_histogram_set_ranges(h, v->data, size);
       gsl_vector_free(v);
       return Data_Wrap_Struct(klass, 0, gsl_histogram_free, h);
@@ -89,7 +93,7 @@ static VALUE rb_gsl_histogram_alloc(int argc, VALUE *argv, VALUE klass)
       CHECK_FIXNUM(argv[1]);
       Data_Get_Struct(argv[0], gsl_vector, v);
       size = FIX2INT(argv[1]);
-      h = gsl_histogram_alloc(size-1);
+      h = gsl_histogram_calloc(size-1);
       gsl_histogram_set_ranges(h, v->data, size);
       return Data_Wrap_Struct(klass, 0, gsl_histogram_free, h);
       break;
@@ -97,26 +101,72 @@ static VALUE rb_gsl_histogram_alloc(int argc, VALUE *argv, VALUE klass)
     break;
   default:
     rb_raise(rb_eArgError, "wrong number of arguments (%d for 1, 2, 3)", argc);
+    break;
   }
 
 }
 
+static VALUE rb_gsl_histogram_alloc_from_file(VALUE klass, VALUE name)
+{
+  char filename[1024], buf[1024];
+  gsl_histogram *h;
+  int nn;
+  size_t n, i;
+  FILE *fp = NULL;
+  double upper;
+  strcpy(filename, STR2CHARPTR(name));
+  sprintf(buf, "wc %s", filename);
+  fp = popen(buf, "r");
+  if (fp == NULL) rb_raise(rb_eIOError, "popen failed.");
+  fgets(buf, 1024, fp);
+  fclose(fp);
+  sscanf(buf, "%d", &nn);
+  n = (size_t) nn;      /* vector length */
+  fp = fopen(filename, "r");
+  if (fp == NULL) rb_raise(rb_eIOError, "cannot open file %s.", filename);
+  h = gsl_histogram_alloc(n);
+  i = 0;
+  while (fgets(buf, 1024, fp)) {
+    sscanf(buf, "%lg %lg %lg", h->range+i, &upper, h->bin+i);
+    i++;
+  }
+  h->range[n] = upper;
+  fclose(fp);
+  return Data_Wrap_Struct(klass, 0, gsl_histogram_free, h);
+  
+}
+
 /* initialization + set uniform ranges (equal spacing from min to max) */
-static VALUE rb_gsl_histogram_alloc_uniform(VALUE klass, VALUE nn, VALUE vmin, 
-					    VALUE vmax)
+static VALUE rb_gsl_histogram_alloc_uniform(int argc, VALUE *argv, VALUE klass)
 {
   gsl_histogram *h = NULL;
   double min, max, tmp;
-  CHECK_FIXNUM(nn);
-  Need_Float(vmin); Need_Float(vmax);
-  min = NUM2DBL(vmin);
-  max = NUM2DBL(vmax);
+  size_t n;
+  switch (argc) {
+  case 3:
+    CHECK_FIXNUM(argv[0]);
+    Need_Float(argv[1]); Need_Float(argv[2]);
+    n = FIX2INT(argv[0]);
+    min = NUM2DBL(argv[1]);
+    max = NUM2DBL(argv[2]);
+    break;
+  case 2:
+    CHECK_FIXNUM(argv[0]);
+    n = FIX2INT(argv[0]);
+    Check_Type(argv[1], T_ARRAY);
+    min = NUM2DBL(rb_ary_entry(argv[1], 0));
+    max = NUM2DBL(rb_ary_entry(argv[1], 1));
+    break;
+  default:
+    rb_raise(rb_eArgError, "wrong number of arguments (%d for 2 or 3)", argc);
+    break;
+  }
   if (min > max) {
     tmp = min;
     min = max;
     max = tmp;
   }
-  h = gsl_histogram_alloc(FIX2INT(nn));
+  h = gsl_histogram_alloc(n);
   gsl_histogram_set_ranges_uniform(h, min, max);
   return Data_Wrap_Struct(klass, 0, gsl_histogram_free, h);
 }
@@ -175,6 +225,7 @@ static VALUE rb_gsl_histogram_calloc_range(int argc, VALUE *argv,  VALUE klass)
     break;
   default:
     rb_raise(rb_eArgError, "wrong number of arguments (%d for 1 or 2)", argc);
+    break;
   }
   h = gsl_histogram_calloc_range(n, v->data);
   return Data_Wrap_Struct(klass, 0, gsl_histogram_free, h);
@@ -235,11 +286,26 @@ static VALUE rb_gsl_histogram_bin(VALUE obj)
   return Data_Wrap_Struct(cgsl_histogram_bin, 0, gsl_vector_view_free, v);
 }
 
-static VALUE rb_gsl_histogram_set_ranges_uniform(VALUE obj, VALUE xmin, VALUE xmax)
+static VALUE rb_gsl_histogram_set_ranges_uniform(int argc, VALUE *argv, VALUE obj)
 {
   gsl_histogram *h = NULL;
+  double xmin, xmax;
+  switch (argc) {
+  case 1:
+    Check_Type(argv[0], T_ARRAY);
+    xmin = NUM2DBL(rb_ary_entry(argv[0], 0));
+    xmax = NUM2DBL(rb_ary_entry(argv[0], 1));
+    break;
+  case 2:
+    xmin = NUM2DBL(argv[0]);
+    xmax = NUM2DBL(argv[1]);
+    break;
+  default:
+    rb_raise(rb_eArgError, "wrong number of arguments (%d for 1 or 2)");
+    break;
+  }
   Data_Get_Struct(obj, gsl_histogram, h);
-  gsl_histogram_set_ranges_uniform(h, NUM2DBL(xmin), NUM2DBL(xmax));
+  gsl_histogram_set_ranges_uniform(h, xmin, xmax);
   return obj;
 }
 
@@ -266,22 +332,47 @@ static VALUE rb_gsl_histogram_clone(VALUE obj)
 static VALUE rb_gsl_histogram_accumulate(int argc, VALUE *argv, VALUE obj)
 {
   gsl_histogram *h = NULL;
-  double x, weight = 1;
+  gsl_vector *v;
+  gsl_vector_int *vi;
+  size_t i;
+  double weight = 1;
+#ifdef HAVE_NARRAY_H
+  double *ptr;
+  size_t size, stride;
+#endif
   switch (argc) {
   case 2:
     Need_Float(argv[1]);
     weight = NUM2DBL(argv[1]);
-    /* no break; */
+    break;
   case 1:
-    Need_Float(argv[0]);
-    x = NUM2DBL(argv[0]);
+    weight = 1;
     break;
   default:
     rb_raise(rb_eArgError, "wrong number of arguments (%d for 1 or 2)", argc);
     break;
   }
   Data_Get_Struct(obj, gsl_histogram, h);
-  gsl_histogram_accumulate(h, x, weight);
+  if (TYPE(argv[0]) == T_ARRAY) {
+    for (i = 0; i < RARRAY(argv[0])->len; i++)
+      gsl_histogram_accumulate(h, NUM2DBL(rb_ary_entry(argv[0], i)), weight);
+  } else if (VECTOR_P(argv[0])) {
+    Data_Get_Struct(argv[0], gsl_vector, v);
+    for (i = 0; i < v->size; i++)
+      gsl_histogram_accumulate(h, gsl_vector_get(v, i), weight);
+  } else if (VECTOR_INT_P(argv[0])) {
+    Data_Get_Struct(argv[0], gsl_vector_int, vi);
+    for (i = 0; i < vi->size; i++)
+      gsl_histogram_accumulate(h, (double)gsl_vector_int_get(vi, i), weight);
+#ifdef HAVE_NARRAY_H
+  } else if (NA_IsNArray(argv[0])) {
+    ptr = get_vector_ptr(argv[0], &stride, &size);
+    for (i = 0; i < size; i++) 
+      gsl_histogram_accumulate(h, ptr[i], weight);
+#endif
+  } else {
+    gsl_histogram_accumulate(h, NUM2DBL(argv[0]), weight);
+  }
   return argv[0];
 }
 
@@ -447,7 +538,7 @@ static VALUE rb_gsl_histogram_normalize(VALUE obj)
 static VALUE rb_gsl_histogram_integral(int argc, VALUE *argv, VALUE obj)
 {
   gsl_histogram *h = NULL;
-  size_t offset = 0, n, i = 0;
+  size_t istart = 0, iend, i = 0;
   double sum = 0.0;
   Data_Get_Struct(obj, gsl_histogram, h);
   switch (argc) {
@@ -456,18 +547,20 @@ static VALUE rb_gsl_histogram_integral(int argc, VALUE *argv, VALUE obj)
     break;
   case 1:
     CHECK_FIXNUM(argv[0]);
-    offset = 0; n = (size_t) FIX2UINT(argv[0]);
+    istart = 0; iend = FIX2INT(argv[0]);
     break;
   case 2:
     CHECK_FIXNUM(argv[0]); CHECK_FIXNUM(argv[1]);
-    offset = (size_t) FIX2UINT(argv[0]);
-    n = (size_t) FIX2UINT(argv[1]);
+    istart = FIX2INT(argv[0]);
+    iend = FIX2INT(argv[1]);
     break;
   default:
     rb_raise(rb_eArgError, "wrong number of arguments (%d for 0-2)", argc);
     break;
   }
-  while (i < n) sum += h->bin[offset+i];
+  if (iend >= h->n) iend = h->n - 1;
+  i = istart;
+  while (i <= iend) sum += h->bin[i++];
   return rb_float_new(sum);
 }
 
@@ -491,6 +584,7 @@ static VALUE rb_gsl_histogram_equal_bins_p(int argc, VALUE *argv, VALUE obj)
     Data_Get_Struct(obj, gsl_histogram, h1);
     CHECK_HISTOGRAM(argv[0]);
     Data_Get_Struct(argv[0], gsl_histogram, h2);
+    break;
   }
   return INT2FIX(gsl_histogram_equal_bins_p(h1, h2));
 }
@@ -515,6 +609,7 @@ static VALUE rb_gsl_histogram_equal_bins_p2(int argc, VALUE *argv, VALUE obj)
     Data_Get_Struct(obj, gsl_histogram, h1);
     CHECK_HISTOGRAM(argv[0]);
     Data_Get_Struct(argv[0], gsl_histogram, h2);
+    break;
   }
   if (gsl_histogram_equal_bins_p(h1, h2)) return Qtrue;
   else return Qfalse;
@@ -527,7 +622,7 @@ static VALUE rb_gsl_histogram_add(VALUE obj, VALUE hh2)
   hnew = gsl_histogram_clone(h1);
   if (HISTOGRAM_P(hh2)) {
     Data_Get_Struct(hh2, gsl_histogram, h2);
-    gsl_histogram_add(hnew, h2);
+    mygsl_histogram_add(hnew, h2);
   } else {
     Need_Float(hh2);
     gsl_histogram_shift(hnew, NUM2DBL(hh2));
@@ -535,6 +630,20 @@ static VALUE rb_gsl_histogram_add(VALUE obj, VALUE hh2)
   return Data_Wrap_Struct(CLASS_OF(obj), 0, gsl_histogram_free, hnew);
 }
 
+static VALUE rb_gsl_histogram_add2(VALUE obj, VALUE hh2)
+{
+  gsl_histogram *h1 = NULL, *h2 = NULL;
+  Data_Get_Struct(obj, gsl_histogram, h1);
+  if (HISTOGRAM_P(hh2)) {
+    Data_Get_Struct(hh2, gsl_histogram, h2);
+    mygsl_histogram_add(h1, h2);
+  } else {
+    Need_Float(hh2);
+    gsl_histogram_shift(h1, NUM2DBL(hh2));
+  }
+  return obj;
+}
+
 static VALUE rb_gsl_histogram_sub(VALUE obj, VALUE hh2)
 {
   gsl_histogram *h1 = NULL, *h2 = NULL, *hnew = NULL;
@@ -542,7 +651,7 @@ static VALUE rb_gsl_histogram_sub(VALUE obj, VALUE hh2)
   hnew = gsl_histogram_clone(h1);
   if (HISTOGRAM_P(hh2)) {
     Data_Get_Struct(hh2, gsl_histogram, h2);
-    gsl_histogram_sub(hnew, h2);
+    mygsl_histogram_sub(hnew, h2);
   } else {
     Need_Float(hh2);
     gsl_histogram_shift(hnew, -NUM2DBL(hh2));
@@ -550,6 +659,20 @@ static VALUE rb_gsl_histogram_sub(VALUE obj, VALUE hh2)
   return Data_Wrap_Struct(CLASS_OF(obj), 0, gsl_histogram_free, hnew);
 }
 
+static VALUE rb_gsl_histogram_sub2(VALUE obj, VALUE hh2)
+{
+  gsl_histogram *h1 = NULL, *h2 = NULL;
+  Data_Get_Struct(obj, gsl_histogram, h1);
+  if (HISTOGRAM_P(hh2)) {
+    Data_Get_Struct(hh2, gsl_histogram, h2);
+    mygsl_histogram_sub(h1, h2);
+  } else {
+    Need_Float(hh2);
+    gsl_histogram_shift(h1, -NUM2DBL(hh2));
+  }
+  return obj;
+}
+
 static VALUE rb_gsl_histogram_mul(VALUE obj, VALUE hh2)
 {
   gsl_histogram *h1 = NULL, *h2 = NULL, *hnew = NULL;
@@ -557,7 +680,7 @@ static VALUE rb_gsl_histogram_mul(VALUE obj, VALUE hh2)
   hnew = gsl_histogram_clone(h1);
   if (HISTOGRAM_P(hh2)) {
     Data_Get_Struct(hh2, gsl_histogram, h2);
-    gsl_histogram_mul(hnew, h2);
+    mygsl_histogram_mul(hnew, h2);
   } else {
     Need_Float(hh2);
     gsl_histogram_scale(hnew, NUM2DBL(hh2));
@@ -565,6 +688,20 @@ static VALUE rb_gsl_histogram_mul(VALUE obj, VALUE hh2)
   return Data_Wrap_Struct(CLASS_OF(obj), 0, gsl_histogram_free, hnew);
 }
 
+static VALUE rb_gsl_histogram_mul2(VALUE obj, VALUE hh2)
+{
+  gsl_histogram *h1 = NULL, *h2 = NULL;
+  Data_Get_Struct(obj, gsl_histogram, h1);
+  if (HISTOGRAM_P(hh2)) {
+    Data_Get_Struct(hh2, gsl_histogram, h2);
+    mygsl_histogram_mul(h1, h2);
+  } else {
+    Need_Float(hh2);
+    gsl_histogram_scale(h1, NUM2DBL(hh2));
+  }
+  return obj;
+}
+
 static VALUE rb_gsl_histogram_div(VALUE obj, VALUE hh2)
 {
   gsl_histogram *h1 = NULL, *h2 = NULL, *hnew = NULL;
@@ -572,7 +709,7 @@ static VALUE rb_gsl_histogram_div(VALUE obj, VALUE hh2)
   hnew = gsl_histogram_clone(h1);
   if (HISTOGRAM_P(hh2)) {
     Data_Get_Struct(hh2, gsl_histogram, h2);
-    gsl_histogram_div(hnew, h2);
+    mygsl_histogram_div(hnew, h2);
   } else {
     Need_Float(hh2);
     gsl_histogram_scale(hnew, 1.0/NUM2DBL(hh2));
@@ -580,6 +717,20 @@ static VALUE rb_gsl_histogram_div(VALUE obj, VALUE hh2)
   return Data_Wrap_Struct(CLASS_OF(obj), 0, gsl_histogram_free, hnew);
 }
 
+static VALUE rb_gsl_histogram_div2(VALUE obj, VALUE hh2)
+{
+  gsl_histogram *h1 = NULL, *h2 = NULL;
+  Data_Get_Struct(obj, gsl_histogram, h1);
+  if (HISTOGRAM_P(hh2)) {
+    Data_Get_Struct(hh2, gsl_histogram, h2);
+    mygsl_histogram_div(h1, h2);
+  } else {
+    Need_Float(hh2);
+    gsl_histogram_scale(h1, 1.0/NUM2DBL(hh2));
+  }
+  return obj;
+}
+
 static VALUE rb_gsl_histogram_scale_bang(int argc, VALUE *argv, VALUE obj)
 {
   gsl_histogram *h = NULL;
@@ -804,6 +955,7 @@ static VALUE rb_gsl_histogram_graph(int argc, VALUE *argv, VALUE obj)
     break;
   default:
     rb_raise(rb_eArgError, "wrong number of arguments (%d for 0 or 1)", argc);
+    break;
   }
   fp = popen(command, "w");
   if (fp == NULL) rb_raise(rb_eIOError, "GNU graph not found.");
@@ -843,6 +995,7 @@ static VALUE rb_gsl_histogram_plot(int argc, VALUE *argv, VALUE obj)
     break;
   default:
     rb_raise(rb_eArgError, "wrong number of arguments (%d for 0 or 1)", argc);
+    break;
   }
   for (i = 0; i < v->n; i++) {
     fprintf(fp, "%e %e\n", v->range[i], v->bin[i]);
@@ -1086,11 +1239,12 @@ static VALUE rb_gsl_histogram_fit_gaussian(int argc, VALUE *argv, VALUE obj)
   mean = gsl_vector_get(s->x, 1);
   height = gsl_vector_get(s->x, 2)*sigma*sqrt(2*M_PI);
   gsl_multifit_covar(s->J, 0.0, covar);
-  errs = sqrt(gsl_matrix_get(covar, 0, 0));
-  errm = sqrt(gsl_matrix_get(covar, 1, 1));
-  errh = sqrt(gsl_matrix_get(covar, 2, 2));
-  chi2 = gsl_blas_dnrm2(s->f);   /* not reduces chi-square */
+  chi2 = gsl_pow_2(gsl_blas_dnrm2(s->f));   /* not reduced chi-square */
   dof = n - p;
+  errs = sqrt(chi2/dof*gsl_matrix_get(covar, 0, 0))/sigma/2;
+  errm = sqrt(chi2/dof*gsl_matrix_get(covar, 1, 1));
+  errh = sqrt(chi2/dof*gsl_matrix_get(covar, 2, 2));
+
   gsl_multifit_fdfsolver_free(s);
   gsl_vector_free(x);
   gsl_matrix_free(covar);
@@ -1100,12 +1254,44 @@ static VALUE rb_gsl_histogram_fit_gaussian(int argc, VALUE *argv, VALUE obj)
 		     rb_float_new(chi2), INT2FIX(dof));
 }
 
+static VALUE rb_gsl_histogram_fit(int argc, VALUE *argv, VALUE obj)
+{
+  char fittype[32];
+  if (argc < 1) rb_raise(rb_eArgError, "too few arguments");
+  Check_Type(argv[0], T_STRING);
+  strcpy(fittype, STR2CSTR(argv[0]));
+  if (str_head_grep(fittype, "exp") == 0) {
+    return rb_gsl_histogram_fit_exponential(argc-1, argv+1, obj);
+  } else if (str_head_grep(fittype, "power") == 0) {
+    return rb_gsl_histogram_fit_power(argc-1, argv+1, obj);
+  } else if (str_head_grep(fittype, "gaus") == 0) {
+    return rb_gsl_histogram_fit_gaussian(argc-1, argv+1, obj);
+  } else {
+    rb_raise(rb_eRuntimeError, 
+	     "unknown fitting type %s (exp, power, gaus expected)", fittype);
+  }
+  return Qnil;
+}
+
 /* Integrate histogram: the two histograms must have the same range and bins. */
-void mygsl_histogram_integrate(const gsl_histogram *h, gsl_histogram *hi)
+void mygsl_histogram_integrate(const gsl_histogram *h, gsl_histogram *hi,
+			       size_t istart, size_t iend)
 {
   size_t i;
-  hi->bin[0] = h->bin[0];
-  for (i = 1; i < h->n; i++) hi->bin[i] = hi->bin[i-1] + h->bin[i];
+  if (iend >= istart) {
+    if (istart < 0) istart = 0;
+    if (iend >= h->n) iend = h->n-1;
+    hi->bin[istart] = h->bin[istart];
+    for (i = istart+1; i <= iend; i++) hi->bin[i] = hi->bin[i-1] + h->bin[i];
+  } else {
+    if (istart >= h->n) istart = h->n-1;
+    if (iend < 0) iend = 0;
+    hi->bin[istart] = h->bin[istart];
+    for (i = istart-1; i >= iend; i--) {
+      hi->bin[i] = hi->bin[i+1] + h->bin[i];
+      if (i == 0) break;
+    }
+  }
 }
 
 void mygsl_histogram_differentiate(const gsl_histogram *hi, gsl_histogram *h)
@@ -1116,11 +1302,12 @@ void mygsl_histogram_differentiate(const gsl_histogram *hi, gsl_histogram *h)
 }
 
 /* Create a histogram integrating the given histogram h */
-gsl_histogram* mygsl_histogram_calloc_integrate(const gsl_histogram *h)
+gsl_histogram* mygsl_histogram_calloc_integrate(const gsl_histogram *h,
+						size_t istart, size_t iend)
 {
   gsl_histogram *hi = NULL;
   hi = gsl_histogram_calloc_range(h->n, h->range);
-  mygsl_histogram_integrate(h, hi);
+  mygsl_histogram_integrate(h, hi, istart, iend);
   return hi;
 }
 
@@ -1132,11 +1319,48 @@ gsl_histogram* mygsl_histogram_calloc_differentiate(const gsl_histogram *hi)
   return h;
 }
 
-static VALUE rb_gsl_histogram_integrate(VALUE obj)
+static VALUE rb_gsl_histogram_integrate(int argc, VALUE *argv, VALUE obj)
 {
   gsl_histogram *h, *hi;
+  size_t istart, iend;
+  int itmp;
   Data_Get_Struct(obj, gsl_histogram, h);
-  hi = mygsl_histogram_calloc_integrate(h);
+  switch (argc) {
+  case 2:
+    istart = FIX2INT(argv[0]);
+    iend = FIX2INT(argv[1]);
+    break;
+  case 1:
+    switch (TYPE(argv[0])) {
+    case T_ARRAY:
+      istart = FIX2INT(rb_ary_entry(argv[0], 0));
+      iend = FIX2INT(rb_ary_entry(argv[0], 1));
+      break;
+    case T_FIXNUM:
+      itmp = FIX2INT(argv[0]);
+      if (itmp == -1) {
+	istart = h->n - 1;
+	iend = 0;
+      } else {
+	istart = 0;
+	iend = h->n - 1;
+      }
+      break;
+    default:
+      rb_raise(rb_eArgError, "wrong argument type %s (Arran or Fixnum expected)",
+	       rb_class2name(CLASS_OF(argv[0])));
+      break;
+    }
+    break;
+  case 0:
+    istart = 0;
+    iend = h->n - 1;
+    break;
+  default:
+    rb_raise(rb_eArgError, "wrong number of arguments (%d for 0-2)", argc);
+    break;
+  }
+  hi = mygsl_histogram_calloc_integrate(h, istart, iend);
   return Data_Wrap_Struct(cgsl_histogram_integ, 0, gsl_histogram_free, hi);
 }
 
@@ -1190,15 +1414,83 @@ static VALUE rb_gsl_histogram_rebin(int argc, VALUE *argv, VALUE obj)
   return Data_Wrap_Struct(cgsl_histogram, 0, gsl_histogram_free, hnew);
 }
 
+static int mygsl_histogram_fread2(FILE * stream, gsl_histogram * h)
+{  
+  double min, max;
+  int status;
+  status = gsl_block_raw_fread(stream, &min, 1, 1);
+  if (status)    return status;  
+  status = gsl_block_raw_fread(stream, &max, 1, 1);
+  if (status)    return status;  
+  gsl_histogram_set_ranges_uniform(h, min, max);
+  status = gsl_block_raw_fread (stream, h->bin, h->n, 1);  
+  if (status)    return status;  
+  return status;
+}
+
+static int mygsl_histogram_fwrite2(FILE * stream, const gsl_histogram * h)
+{  
+  int status;
+  status = gsl_block_raw_fwrite (stream, h->range, 1, 1);
+  if (status)    return status;  
+  status = gsl_block_raw_fwrite (stream, h->range+h->n, 1, 1);
+  if (status)    return status;  
+  status = gsl_block_raw_fwrite (stream, h->bin, h->n, 1);  
+  return status;
+}
+
+static VALUE rb_gsl_histogram_fwrite2(VALUE obj, VALUE io)
+{
+  gsl_histogram *h = NULL;
+  FILE *f;
+  int status, flag = 0;
+  Data_Get_Struct(obj, gsl_histogram, h);
+  f = rb_gsl_open_writefile(io, &flag);
+  status = mygsl_histogram_fwrite2(f, h);
+  if (flag == 1) fclose(f);
+  return INT2FIX(status);
+}
+
+static VALUE rb_gsl_histogram_fread2(VALUE obj, VALUE io)
+{
+  gsl_histogram *h = NULL;
+  FILE *f;
+  int status, flag = 0;
+  Data_Get_Struct(obj, gsl_histogram, h);
+  f = rb_gsl_open_readfile(io, &flag);
+  status = mygsl_histogram_fread2(f, h);
+  if (flag == 1) fclose(f);
+  return INT2FIX(status);
+}
+
+static gsl_histogram* mygsl_histogram_calloc_reverse(const gsl_histogram *h)
+{
+  gsl_histogram *hnew;
+  size_t i, n;
+  hnew = gsl_histogram_alloc(h->n);
+  n = h->n;
+  for (i = 0; i <= n; i++) hnew->range[i] = h->range[n-i];
+  for (i = 0; i < n; i++) hnew->bin[i] = h->bin[n-1-i];
+  return hnew;
+}
+
+static VALUE rb_gsl_histogram_reverse(VALUE obj)
+{
+  gsl_histogram *h, *hnew;
+  Data_Get_Struct(obj, gsl_histogram, h);
+  hnew = mygsl_histogram_calloc_reverse(h);
+  return Data_Wrap_Struct(cgsl_histogram, 0, gsl_histogram_free, hnew);
+}
+
 void Init_gsl_histogram(VALUE module)
 {
   VALUE cgsl_histogram_pdf;
 
-  cgsl_histogram = rb_define_class_under(module, "Histogram", rb_cObject);
+  cgsl_histogram = rb_define_class_under(module, "Histogram", cGSL_Object);
   cgsl_histogram_range = rb_define_class_under(cgsl_histogram, "Range", 
 					       cgsl_vector_view_ro);
   cgsl_histogram_bin = rb_define_class_under(cgsl_histogram, "Bin", 
-					     cgsl_vector_view_ro);
+					     cgsl_vector_view);
   cgsl_histogram_integ = rb_define_class_under(cgsl_histogram, "Integral", 
 					       cgsl_histogram);
 
@@ -1208,9 +1500,9 @@ void Init_gsl_histogram(VALUE module)
   rb_define_singleton_method(cgsl_histogram, "[]", rb_gsl_histogram_alloc, -1);
 
   rb_define_singleton_method(cgsl_histogram, "alloc_uniform", 
-			     rb_gsl_histogram_alloc_uniform, -1);
+			     rb_gsl_histogram_alloc_uniform, 3);
   rb_define_singleton_method(cgsl_histogram, "new_uniform", 
-			     rb_gsl_histogram_alloc_uniform, -1);
+			     rb_gsl_histogram_alloc_uniform, 3);
 
   rb_define_singleton_method(cgsl_histogram, "alloc_with_min_max_step", 
 			     rb_gsl_histogram_alloc_with_min_max_step, 3);
@@ -1230,7 +1522,7 @@ void Init_gsl_histogram(VALUE module)
   rb_define_method(cgsl_histogram, "range", rb_gsl_histogram_range, 0);
   rb_define_method(cgsl_histogram, "bin", rb_gsl_histogram_bin, 0);
   rb_define_method(cgsl_histogram, "set_ranges_uniform", 
-		   rb_gsl_histogram_set_ranges_uniform, 2);
+		   rb_gsl_histogram_set_ranges_uniform, -1);
   rb_define_singleton_method(cgsl_histogram, "memcpy", rb_gsl_histogram_memcpy, 2);
   rb_define_method(cgsl_histogram, "clone", rb_gsl_histogram_clone, 0);
   rb_define_alias(cgsl_histogram, "duplicate", "clone");
@@ -1282,6 +1574,11 @@ void Init_gsl_histogram(VALUE module)
   rb_define_method(cgsl_histogram, "div", rb_gsl_histogram_div, 1);
   rb_define_alias(cgsl_histogram, "/", "div");
 
+  rb_define_method(cgsl_histogram, "add!", rb_gsl_histogram_add2, 1);
+  rb_define_method(cgsl_histogram, "sub!", rb_gsl_histogram_sub2, 1);
+  rb_define_method(cgsl_histogram, "mul!", rb_gsl_histogram_mul2, 1);
+  rb_define_method(cgsl_histogram, "div!", rb_gsl_histogram_div2, 1);
+
   rb_define_method(cgsl_histogram, "scale!", rb_gsl_histogram_scale_bang, -1);
   rb_define_method(cgsl_histogram, "scale", rb_gsl_histogram_scale, -1);
   rb_define_method(cgsl_histogram, "shift!", rb_gsl_histogram_shift, 1);
@@ -1289,12 +1586,14 @@ void Init_gsl_histogram(VALUE module)
 
   rb_define_method(cgsl_histogram, "fwrite", rb_gsl_histogram_fwrite, 1);
   rb_define_method(cgsl_histogram, "fread", rb_gsl_histogram_fread, 1);
+  rb_define_method(cgsl_histogram, "fwrite2", rb_gsl_histogram_fwrite2, 1);
+  rb_define_method(cgsl_histogram, "fread2", rb_gsl_histogram_fread2, 1);
   rb_define_method(cgsl_histogram, "fprintf", rb_gsl_histogram_fprintf, -1);
   rb_define_method(cgsl_histogram, "printf", rb_gsl_histogram_printf, -1);
   rb_define_method(cgsl_histogram, "fscanf", rb_gsl_histogram_fscanf, 1);
   rb_define_method(cgsl_histogram, "print", rb_gsl_histogram_print, 0);
 
-  cgsl_histogram_pdf = rb_define_class_under(cgsl_histogram, "Pdf", rb_cObject);
+  cgsl_histogram_pdf = rb_define_class_under(cgsl_histogram, "Pdf", cGSL_Object);
   rb_define_singleton_method(cgsl_histogram_pdf, "alloc", 
 			     rb_gsl_histogram_pdf_alloc, 1);
   rb_define_singleton_method(cgsl_histogram_pdf, "new", 
@@ -1319,8 +1618,11 @@ void Init_gsl_histogram(VALUE module)
 		   rb_gsl_histogram_fit_exponential, -1);
   rb_define_method(cgsl_histogram, "fit_power", 
 		   rb_gsl_histogram_fit_power, -1);
+  rb_define_method(cgsl_histogram, "fit", 
+		   rb_gsl_histogram_fit, -1);
 
-  rb_define_method(cgsl_histogram, "integrate", rb_gsl_histogram_integrate, 0);
+  rb_define_method(cgsl_histogram, "integrate", rb_gsl_histogram_integrate, -1);
+  rb_undef_method(cgsl_histogram_integ, "integrate");
   rb_define_method(cgsl_histogram_integ, "differentiate", 
 		   rb_gsl_histogram_differentiate, 0);
   rb_define_alias(cgsl_histogram_integ, "diff", "differentiate");
@@ -1330,4 +1632,6 @@ void Init_gsl_histogram(VALUE module)
 
   rb_define_method(cgsl_histogram, "rebin", rb_gsl_histogram_rebin, -1);
   rb_define_alias(cgsl_histogram, "mergebin", "rebin");
+
+  rb_define_method(cgsl_histogram, "reverse", rb_gsl_histogram_reverse, 0);
 }
