@@ -1272,6 +1272,19 @@ static VALUE FUNCTION(rb_gsl_matrix,collect)(VALUE obj)
   return Data_Wrap_Struct(GSL_TYPE(cgsl_matrix), 0, FUNCTION(gsl_matrix,free), mnew);
 }
 
+static VALUE FUNCTION(rb_gsl_matrix,collect_bang)(VALUE obj)
+{
+  GSL_TYPE(gsl_matrix) *m = NULL;
+  size_t i, j;
+  Data_Get_Struct(obj, GSL_TYPE(gsl_matrix), m);
+  for (i = 0; i < m->size1; i++) {
+    for (j = 0; j < m->size2; j++) {
+      FUNCTION(gsl_matrix,set)(m, i, j, NUMCONV(rb_yield(C_TO_VALUE(FUNCTION(gsl_matrix,get)(m, i, j)))));
+    }
+  }
+  return obj;
+}
+
 static VALUE FUNCTION(rb_gsl_matrix,upper)(VALUE obj)
 {
   GSL_TYPE(gsl_matrix) *m = NULL, *mnew;
@@ -1786,6 +1799,21 @@ static VALUE FUNCTION(rb_gsl_matrix,finite)(VALUE obj)
   return FUNCTION(rb_gsl_matrix,test)(obj, gsl_finite);
 }
 
+static VALUE FUNCTION(rb_gsl_matrix,abs)(VALUE obj)
+{
+  GSL_TYPE(gsl_matrix) *m, *mnew;
+  size_t i, j;
+  Data_Get_Struct(obj, GSL_TYPE(gsl_matrix), m);
+  mnew = FUNCTION(gsl_matrix,alloc)(m->size1, m->size2);
+  for (i = 0; i < m->size1; i++) {
+    for (j = 0; j < m->size2; j++) {
+      FUNCTION(gsl_matrix,set)(mnew, i, j, (BASE) fabs(FUNCTION(gsl_matrix,get)(m, i, j)));
+    }
+  }
+
+  return Data_Wrap_Struct(GSL_TYPE(cgsl_matrix), 0, FUNCTION(gsl_matrix,free), mnew);
+}
+
 void FUNCTION(Init_gsl_matrix,init)(VALUE module)
 {
   rb_define_singleton_method(GSL_TYPE(cgsl_matrix), "new", FUNCTION(rb_gsl_matrix,new), -1);
@@ -1974,6 +2002,8 @@ void FUNCTION(Init_gsl_matrix,init)(VALUE module)
 
   rb_define_method(GSL_TYPE(cgsl_matrix), "collect", 
 		   FUNCTION(rb_gsl_matrix,collect), 0);
+  rb_define_method(GSL_TYPE(cgsl_matrix), "collect!", 
+		   FUNCTION(rb_gsl_matrix,collect_bang), 0);
 #ifdef HAVE_GSL_TENSOR_GSL_TENSOR_H
   rb_define_method(GSL_TYPE(cgsl_matrix), "to_tensor", 
 		   FUNCTION(rb_gsl_matrix,to_tensor), 0);
@@ -2038,6 +2068,8 @@ void FUNCTION(Init_gsl_matrix,init)(VALUE module)
   rb_define_method(GSL_TYPE(cgsl_matrix), "isinf", FUNCTION(rb_gsl_matrix,isinf), 0);
   rb_define_method(GSL_TYPE(cgsl_matrix), "finite", FUNCTION(rb_gsl_matrix,finite), 0);
 
+  rb_define_method(GSL_TYPE(cgsl_matrix), "abs", FUNCTION(rb_gsl_matrix,abs), 0);
+
 }
 
 #undef NUMCONV
