@@ -20,8 +20,8 @@
       rb_class2name(CLASS_OF(x)));
 #endif
 
-#ifndef CHECK_FUNCTION_FDF
-#define CHECK_FUNCTION_FDF(x) if(CLASS_OF(x)!=cgsl_multifit_function_fdf)\
+#ifndef CHECK_MULTIFIT_FUNCTION_FDF
+#define CHECK_MULTIFIT_FUNCTION_FDF(x) if(CLASS_OF(x)!=cgsl_multifit_function_fdf)\
       rb_raise(rb_eTypeError,\
       "wrong argument type %s (GSL::MultiFit::Workspace expected)",\
       rb_class2name(CLASS_OF(x)));
@@ -259,7 +259,7 @@ static VALUE rb_gsl_multifit_fdfsolver_set(VALUE obj, VALUE ff, VALUE xx)
   gsl_multifit_function_fdf *f = NULL;
   gsl_vector *x = NULL;
   int status;
-  CHECK_FUNCTION_FDF(ff);
+  CHECK_MULTIFIT_FUNCTION_FDF(ff);
   Data_Get_Struct(obj, gsl_multifit_fdfsolver, solver);
   Data_Get_Struct(ff, gsl_multifit_function_fdf, f);
   Data_Get_Vector(xx, x);
@@ -1739,6 +1739,6 @@ void Init_gsl_multifit(VALUE module)
 #ifdef CHECK_WORKSPACE
 #undef CHECK_WORKSPACE
 #endif
-#ifdef CHECK_FUNCTION_FDF
-#undef CHECK_FUNCTION_FDF
+#ifdef CHECK_MULTIFIT_FUNCTION_FDF
+#undef CHECK_MULTIFIT_FUNCTION_FDF
 #endif
