@@ -122,7 +122,9 @@ static const gsl_rng_type* get_gsl_rng_type_name(char *name)
 #endif
 #ifdef GSL_1_2_LATER
   else if (str_tail_grep(name, "mt19937_1999") == 0) return gsl_rng_mt19937_1999;
+  else if (str_tail_grep(name, "mt19937-1999") == 0) return gsl_rng_mt19937_1999;
   else if (str_tail_grep(name, "mt19937_1998") == 0) return gsl_rng_mt19937_1998;  
+  else if (str_tail_grep(name, "mt19937-1998") == 0) return gsl_rng_mt19937_1998;  
   else if (str_tail_grep(name, "taus113") == 0) return gsl_rng_taus113;
   else if (str_tail_grep(name, "taus2") == 0) return gsl_rng_taus2;
 #endif
@@ -145,36 +147,36 @@ static const gsl_rng_type* get_gsl_rng_type_name(char *name)
   else if (str_tail_grep(name, "random64_libc5") == 0) return gsl_rng_random64_libc5;
   else if (str_tail_grep(name, "random128_libc5") == 0) return gsl_rng_random128_libc5;
   else if (str_tail_grep(name, "random256_libc5") == 0) return gsl_rng_random256_libc5;  
-  else if (str_tail_grep(name, "random_libc5") == 0) return gsl_rng_random_libc5;
-  else if (str_tail_grep(name, "random8_libc5") == 0) return gsl_rng_random8_libc5;
-  else if (str_tail_grep(name, "random32_libc5") == 0) return gsl_rng_random32_libc5;
-  else if (str_tail_grep(name, "random64_libc5") == 0) return gsl_rng_random64_libc5;
-  else if (str_tail_grep(name, "random128_libc5") == 0) return gsl_rng_random128_libc5;
-  else if (str_tail_grep(name, "random256_libc5") == 0) return gsl_rng_random256_libc5;
-  else if (str_tail_grep(name, "random_glibc2") == 0) return gsl_rng_random_glibc2;
-  else if (str_tail_grep(name, "random8_glibc2") == 0) return gsl_rng_random8_glibc2;
-  else if (str_tail_grep(name, "random32_glibc2") == 0) return gsl_rng_random32_glibc2;
-  else if (str_tail_grep(name, "random64_glibc2") == 0) return gsl_rng_random64_glibc2;
-  else if (str_tail_grep(name, "random128_glibc2") == 0) return gsl_rng_random128_glibc2;
-  else if (str_tail_grep(name, "random256_glibc2") == 0) return gsl_rng_random256_glibc2;
+  else if (str_tail_grep(name, "random-libc5") == 0) return gsl_rng_random_libc5;
+  else if (str_tail_grep(name, "random8-libc5") == 0) return gsl_rng_random8_libc5;
+  else if (str_tail_grep(name, "random32-libc5") == 0) return gsl_rng_random32_libc5;
+  else if (str_tail_grep(name, "random64-libc5") == 0) return gsl_rng_random64_libc5;
+  else if (str_tail_grep(name, "random128-libc5") == 0) return gsl_rng_random128_libc5;
+  else if (str_tail_grep(name, "random256-libc5") == 0) return gsl_rng_random256_libc5;
   else if (str_tail_grep(name, "random_glibc2") == 0) return gsl_rng_random_glibc2;
   else if (str_tail_grep(name, "random8_glibc2") == 0) return gsl_rng_random8_glibc2;
   else if (str_tail_grep(name, "random32_glibc2") == 0) return gsl_rng_random32_glibc2;
   else if (str_tail_grep(name, "random64_glibc2") == 0) return gsl_rng_random64_glibc2;
   else if (str_tail_grep(name, "random128_glibc2") == 0) return gsl_rng_random128_glibc2;
   else if (str_tail_grep(name, "random256_glibc2") == 0) return gsl_rng_random256_glibc2;
+  else if (str_tail_grep(name, "random-glibc2") == 0) return gsl_rng_random_glibc2;
+  else if (str_tail_grep(name, "random8-glibc2") == 0) return gsl_rng_random8_glibc2;
+  else if (str_tail_grep(name, "random32-glibc2") == 0) return gsl_rng_random32_glibc2;
+  else if (str_tail_grep(name, "random64-glibc2") == 0) return gsl_rng_random64_glibc2;
+  else if (str_tail_grep(name, "random128-glibc2") == 0) return gsl_rng_random128_glibc2;
+  else if (str_tail_grep(name, "random256-glibc2") == 0) return gsl_rng_random256_glibc2;
   else if (str_tail_grep(name, "random_bsd") == 0) return gsl_rng_random_bsd;
   else if (str_tail_grep(name, "random8_bsd") == 0) return gsl_rng_random8_bsd;
   else if (str_tail_grep(name, "random32_bsd") == 0) return gsl_rng_random32_bsd;
   else if (str_tail_grep(name, "random64_bsd") == 0) return gsl_rng_random64_bsd;
   else if (str_tail_grep(name, "random128_bsd") == 0) return gsl_rng_random128_bsd;
   else if (str_tail_grep(name, "random256_bsd") == 0) return gsl_rng_random256_bsd;
-  else if (str_tail_grep(name, "random_bsd") == 0) return gsl_rng_random_bsd;
-  else if (str_tail_grep(name, "random8_bsd") == 0) return gsl_rng_random8_bsd;
-  else if (str_tail_grep(name, "random32_bsd") == 0) return gsl_rng_random32_bsd;
-  else if (str_tail_grep(name, "random64_bsd") == 0) return gsl_rng_random64_bsd;
-  else if (str_tail_grep(name, "random128_bsd") == 0) return gsl_rng_random128_bsd;
-  else if (str_tail_grep(name, "random256_bsd") == 0) return gsl_rng_random256_bsd;
+  else if (str_tail_grep(name, "random-bsd") == 0) return gsl_rng_random_bsd;
+  else if (str_tail_grep(name, "random8-bsd") == 0) return gsl_rng_random8_bsd;
+  else if (str_tail_grep(name, "random32-bsd") == 0) return gsl_rng_random32_bsd;
+  else if (str_tail_grep(name, "random64-bsd") == 0) return gsl_rng_random64_bsd;
+  else if (str_tail_grep(name, "random128-bsd") == 0) return gsl_rng_random128_bsd;
+  else if (str_tail_grep(name, "random256-bsd") == 0) return gsl_rng_random256_bsd;
   else if (str_tail_grep(name, "rand48") == 0) return gsl_rng_rand48;
   else if (str_tail_grep(name, "ran0") == 0) return gsl_rng_ran0;
   else if (str_tail_grep(name, "ran1") == 0) return gsl_rng_ran1;
@@ -195,9 +197,17 @@ static const gsl_rng_type* get_gsl_rng_type_name(char *name)
 #ifdef HAVE_RNGEXTRA_RNGEXTRA_H
   else if (str_tail_grep(name, "rngextra_rng1") == 0) return rngextra_rng1;
   else if (str_tail_grep(name, "rngextra_rng2") == 0) return rngextra_rng2;
+  else if (str_tail_grep(name, "rngextra-rng1") == 0) return rngextra_rng1;
+  else if (str_tail_grep(name, "rngextra-rng2") == 0) return rngextra_rng2;
 #else
   else if (str_tail_grep(name, "rngextra_rng1")*str_tail_grep(name, "rngextra_rng2") == 0) 
     rb_raise(rb_eNotImpError, "Install the rngextra package found at <http://www.network-theory.co.uk/download/rngextra/>.");
+  else if (str_tail_grep(name, "rngextra_rng2")*str_tail_grep(name, "rngextra_rng2") == 0) 
+    rb_raise(rb_eNotImpError, "Install the rngextra package found at <http://www.network-theory.co.uk/download/rngextra/>.");
+  else if (str_tail_grep(name, "rngextra-rng1")*str_tail_grep(name, "rngextra_rng2") == 0) 
+    rb_raise(rb_eNotImpError, "Install the rngextra package found at <http://www.network-theory.co.uk/download/rngextra/>.");
+  else if (str_tail_grep(name, "rngextra-rng2")*str_tail_grep(name, "rngextra_rng2") == 0) 
+    rb_raise(rb_eNotImpError, "Install the rngextra package found at <http://www.network-theory.co.uk/download/rngextra/>.");
 #endif
   else
     rb_raise(rb_eArgError, "unknown generator type \"%s\"", name);
