@@ -315,7 +315,7 @@ static VALUE rb_gsl_vector_complex_each(VALUE obj)
   size = v->size;
   for (i = 0; i < size; i++) {
     zp = GSL_COMPLEX_AT(v, i);
-    ztmp = *zp;
+    ztmp = *zp;  /* create a temporal copy of *zp */
     rb_yield(Data_Wrap_Struct(cgsl_complex, 0, NULL, zp2));
   }
   return obj;
@@ -332,6 +332,47 @@ static VALUE rb_gsl_vector_complex_each_index(VALUE obj)
   return obj;
 }
 
+static VALUE rb_gsl_vector_complex_collect(VALUE obj)
+{
+  gsl_vector_complex *v = NULL, *vnew;
+  size_t size;
+  gsl_complex *zp = NULL, ztmp, *zp2 = &ztmp, *zp3;
+  size_t i;
+  VALUE a;
+  Data_Get_Struct(obj, gsl_vector_complex, v);
+  vnew = gsl_vector_complex_alloc(v->size);
+  size = v->size;
+  for (i = 0; i < size; i++) {
+    zp = GSL_COMPLEX_AT(v, i);
+    ztmp = *zp;
+    a = rb_yield(Data_Wrap_Struct(cgsl_complex, 0, NULL, zp2));
+    CHECK_COMPLEX(a);
+    Data_Get_Struct(a, gsl_complex, zp3);
+    gsl_vector_complex_set(vnew, i, *zp3);
+  }
+  return Data_Wrap_Struct(cgsl_vector_complex, 0, gsl_vector_complex_free, vnew);
+}
+
+static VALUE rb_gsl_vector_complex_collect_bang(VALUE obj)
+{
+  gsl_vector_complex *v = NULL;
+  size_t size;
+  gsl_complex *zp = NULL, ztmp, *zp2 = &ztmp, *zp3;
+  size_t i;
+  VALUE a;
+  Data_Get_Struct(obj, gsl_vector_complex, v);
+  size = v->size;
+  for (i = 0; i < size; i++) {
+    zp = GSL_COMPLEX_AT(v, i);
+    ztmp = *zp;
+    a = rb_yield(Data_Wrap_Struct(cgsl_complex, 0, NULL, zp2));
+    CHECK_COMPLEX(a);
+    Data_Get_Struct(a, gsl_complex, zp3);
+    gsl_vector_complex_set(v, i, *zp3);
+  }
+  return obj;
+}
+
 static VALUE rb_gsl_vector_complex_set_zero(VALUE obj)
 {
   gsl_vector_complex *v = NULL;
@@ -1392,6 +1433,8 @@ void Init_gsl_vector_complex(VALUE module)
 
   rb_define_method(cgsl_vector_complex, "each", rb_gsl_vector_complex_each, 0);
   rb_define_method(cgsl_vector_complex, "each_index", rb_gsl_vector_complex_each_index, 0);
+  rb_define_method(cgsl_vector_complex, "collect", rb_gsl_vector_complex_collect, 0);
+  rb_define_method(cgsl_vector_complex, "collect!", rb_gsl_vector_complex_collect_bang, 0);
 
   rb_define_method(cgsl_vector_complex, "set_zero", rb_gsl_vector_complex_set_zero, 0);
   rb_define_method(cgsl_vector_complex, "set_basis", rb_gsl_vector_complex_set_basis, 1);
