@@ -998,6 +998,51 @@ static VALUE rb_gsl_vector_round(VALUE obj)
 }
 #endif
 
+static VALUE rb_gsl_vector_dB(VALUE obj)
+{
+  gsl_vector *v, *vnew;
+  double x;
+  size_t i;
+  Data_Get_Struct(obj, gsl_vector, v);
+  vnew = gsl_vector_alloc(v->size);
+  for (i = 0; i < v->size; i++) {
+    x = gsl_vector_get(v, i);
+    if (x <= 0.0) rb_raise(rb_eRuntimeError, "negative value found.\n");
+    gsl_vector_set(vnew, i, 20*log(x));
+  }
+  return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, vnew);
+}
+
+static VALUE rb_gsl_vector_sin(VALUE obj)
+{
+  return rb_gsl_sf_eval1(sin, obj);
+}
+
+static VALUE rb_gsl_vector_cos(VALUE obj)
+{
+  return rb_gsl_sf_eval1(cos, obj);
+}
+
+static VALUE rb_gsl_vector_tan(VALUE obj)
+{
+  return rb_gsl_sf_eval1(tan, obj);
+}
+
+static VALUE rb_gsl_vector_exp(VALUE obj)
+{
+  return rb_gsl_sf_eval1(exp, obj);
+}
+
+static VALUE rb_gsl_vector_log(VALUE obj)
+{
+  return rb_gsl_sf_eval1(log, obj);
+}
+
+static VALUE rb_gsl_vector_log10(VALUE obj)
+{
+  return rb_gsl_sf_eval1(log10, obj);
+}
+
 void Init_gsl_vector(VALUE module)
 {
   rb_define_singleton_method(cgsl_vector, "linspace", rb_gsl_vector_linspace, -1);
@@ -1060,5 +1105,14 @@ void Init_gsl_vector(VALUE module)
   rb_define_method(cgsl_vector, "round", rb_gsl_vector_round, 0);
 #endif
 
+  rb_define_method(cgsl_vector, "dB", rb_gsl_vector_dB, 0);
+
+  rb_define_method(cgsl_vector, "sin", rb_gsl_vector_sin, 0);
+  rb_define_method(cgsl_vector, "cos", rb_gsl_vector_cos, 0);
+  rb_define_method(cgsl_vector, "tan", rb_gsl_vector_tan, 0);
+  rb_define_method(cgsl_vector, "exp", rb_gsl_vector_exp, 0);
+  rb_define_method(cgsl_vector, "log", rb_gsl_vector_log, 0);
+  rb_define_method(cgsl_vector, "log10", rb_gsl_vector_log10, 0);
+
   Init_gsl_vector_init(module);
 }
