@@ -1423,6 +1423,18 @@ static VALUE FUNCTION(rb_gsl_vector,collect)(VALUE obj)
   return Data_Wrap_Struct(GSL_TYPE(cgsl_vector), 0, FUNCTION(gsl_vector,free), vnew);
 }
 
+/* 2004/May/03 */
+static VALUE FUNCTION(rb_gsl_vector,collect_bang)(VALUE obj)
+{
+  GSL_TYPE(gsl_vector) *v = NULL;
+  size_t i;
+  Data_Get_Struct(obj, GSL_TYPE(gsl_vector), v);
+  for (i = 0; i < v->size; i++) {
+    FUNCTION(gsl_vector,set)(v, i, NUMCONV(rb_yield(C_TO_VALUE(FUNCTION(gsl_vector,get)(v, i)))));
+  }
+  return obj;
+}
+
 GSL_TYPE(gsl_vector)* FUNCTION(mygsl_vector,mul_matrix)(GSL_TYPE(gsl_vector) *v,
 							GSL_TYPE(gsl_matrix) *m)
 {
@@ -2683,6 +2695,36 @@ static VALUE FUNCTION(rb_gsl_vector,where2)(VALUE obj)
   return rb_ary_new3(2, vv1, vv2);
 }
 
+static VALUE FUNCTION(rb_gsl_vector,op_inplace)(VALUE vv1, VALUE vv2, 
+						int (*f)(GSL_TYPE(gsl_vector)*, const GSL_TYPE(gsl_vector)*))
+{
+  GSL_TYPE(gsl_vector) *v1, *v2;
+  Data_Get_Struct(vv1, GSL_TYPE(gsl_vector), v1);
+  Data_Get_Struct(vv2, GSL_TYPE(gsl_vector), v2);
+  (*f)(v1, v2);
+  return vv1;
+}
+
+static VALUE FUNCTION(rb_gsl_vector,add_inplace)(VALUE vv1, VALUE vv2)
+{
+  return FUNCTION(rb_gsl_vector,op_inplace)(vv1, vv2, FUNCTION(gsl_vector,add));
+}
+
+static VALUE FUNCTION(rb_gsl_vector,sub_inplace)(VALUE vv1, VALUE vv2)
+{
+  return FUNCTION(rb_gsl_vector,op_inplace)(vv1, vv2, FUNCTION(gsl_vector,sub));
+}
+
+static VALUE FUNCTION(rb_gsl_vector,mul_inplace)(VALUE vv1, VALUE vv2)
+{
+  return FUNCTION(rb_gsl_vector,op_inplace)(vv1, vv2, FUNCTION(gsl_vector,mul));
+}
+
+static VALUE FUNCTION(rb_gsl_vector,div_inplace)(VALUE vv1, VALUE vv2)
+{
+  return FUNCTION(rb_gsl_vector,op_inplace)(vv1, vv2, FUNCTION(gsl_vector,div));
+}
+
 void FUNCTION(Init_gsl_vector,init)(VALUE module)
 {
   rb_define_singleton_method(GSL_TYPE(cgsl_vector), "new", 
@@ -2872,6 +2914,8 @@ void FUNCTION(Init_gsl_vector,init)(VALUE module)
   /*****/
   rb_define_method(GSL_TYPE(cgsl_vector), "collect", 
 		   FUNCTION(rb_gsl_vector,collect), 0);
+  rb_define_method(GSL_TYPE(cgsl_vector), "collect!", 
+		   FUNCTION(rb_gsl_vector,collect_bang), 0);
 
   rb_define_method(GSL_TYPE(cgsl_vector), "to_m", FUNCTION(rb_gsl_vector,to_m), 2);
   rb_define_method(GSL_TYPE(cgsl_vector), "to_m_diagonal", 
@@ -2975,6 +3019,11 @@ void FUNCTION(Init_gsl_vector,init)(VALUE module)
 
   rb_define_method(GSL_TYPE(cgsl_vector), "where", FUNCTION(rb_gsl_vector,where), 0);
   rb_define_method(GSL_TYPE(cgsl_vector), "where2", FUNCTION(rb_gsl_vector,where2), 0);
+
+  rb_define_method(GSL_TYPE(cgsl_vector), "add!", FUNCTION(rb_gsl_vector,add_inplace), 1);
+  rb_define_method(GSL_TYPE(cgsl_vector), "sub!", FUNCTION(rb_gsl_vector,sub_inplace), 1);
+  rb_define_method(GSL_TYPE(cgsl_vector), "mul!", FUNCTION(rb_gsl_vector,mul_inplace), 1);
+  rb_define_method(GSL_TYPE(cgsl_vector), "div!", FUNCTION(rb_gsl_vector,div_inplace), 1);
 }
 
 #undef NUMCONV
