@@ -18,6 +18,10 @@
 #include <gsl/gsl_wavelet2d.h>
 #endif
 
+#ifdef HAVE_NARRAY_H
+#include "narray.h"
+#endif
+
 #ifndef CHECK_WAVELET
 #define CHECK_WAVELET(x) if(!rb_obj_is_kind_of(x,cgsl_wavelet))\
     rb_raise(rb_eTypeError, "wrong argument type (Wavelet expected)");
@@ -28,6 +32,11 @@
     rb_raise(rb_eTypeError, "wrong argument type (Wavelet::Workspace expected)");
 #endif
 
+enum RB_GSL_DWT {
+  RB_GSL_DWT_COPY,
+  RB_GSL_DWT_INPLACE,
+};
+
 static VALUE cgsl_wavelet;
 
 enum {
@@ -165,23 +174,34 @@ static VALUE rb_gsl_wavelet_workspace_new(VALUE klass, VALUE nn)
   return Data_Wrap_Struct(klass, 0, gsl_wavelet_workspace_free, wspace);
 }
 
+static VALUE rb_gsl_wavelet2d_trans(int argc, VALUE *argv, VALUE obj,
+				    int (*trans)(const gsl_wavelet *, 
+						 gsl_matrix *,
+						 gsl_wavelet_workspace *),
+				    int sss);
 static VALUE rb_gsl_wavelet2d(int argc, VALUE *argv, VALUE obj,
 			      int (*trans)(const gsl_wavelet *, 
 					   gsl_matrix *, 
 					   gsl_wavelet_direction, 
-					   gsl_wavelet_workspace *));
-static VALUE rb_gsl_wavelet2d_trans(int argc, VALUE *argv, VALUE obj,
-					 int (*trans)(const gsl_wavelet *, 
-						      gsl_matrix *,
-						      gsl_wavelet_workspace *));
+					   gsl_wavelet_workspace *),
+			      int sss);
 
-static VALUE rb_gsl_wavelet_transform(int argc, VALUE *argv, VALUE obj)
+static VALUE rb_gsl_wavelet_transform0(int argc, VALUE *argv, VALUE obj,
+				       int sss)
 {
   gsl_wavelet *w = NULL;
-  gsl_vector *v = NULL;
+  gsl_vector *v = NULL, *vnew;
   gsl_wavelet_direction dir = forward;
   gsl_wavelet_workspace *work = NULL;
   int itmp, flag = 0, status;
+  double *ptr1, *ptr2;
+  size_t n, stride;
+  int naflag = 0;
+  VALUE ary, ret;
+#ifdef HAVE_NARRAY_H
+  struct NARRAY *na1 = NULL;
+#endif
+
   switch (TYPE(obj)) {
   case T_MODULE:
   case T_CLASS:
@@ -191,12 +211,27 @@ static VALUE rb_gsl_wavelet_transform(int argc, VALUE *argv, VALUE obj)
 
     if (MATRIX_P(argv[1])) {
       return rb_gsl_wavelet2d(argc, argv, obj,
-			      gsl_wavelet2d_transform_matrix);
+			      gsl_wavelet2d_transform_matrix, sss);
+    }
+    if (VECTOR_P(argv[1])) {
+      Data_Get_Struct(argv[0], gsl_wavelet, w);
+      Data_Get_Struct(argv[1], gsl_vector, v);
+      ret = argv[1];
+      ptr1 = v->data;
+      n = v->size;
+      stride = v->stride;
+#ifdef HAVE_NARRAY_H
+    } else if (NA_IsNArray(argv[1])) {
+      GetNArray(argv[1], na1);
+      ret = argv[1];
+      ptr1 = (double*) na1->ptr;
+      n = na1->total;
+      naflag = 1;
+      stride = 1;
+#endif
+    } else {
+      rb_raise(rb_eTypeError, "wrong argument type (Vector expected)");
     }
-
-    CHECK_VECTOR(argv[1]);
-    Data_Get_Struct(argv[0], gsl_wavelet, w);
-    Data_Get_Struct(argv[1], gsl_vector, v);
     itmp = 2;
     break;
   default:
@@ -204,16 +239,47 @@ static VALUE rb_gsl_wavelet_transform(int argc, VALUE *argv, VALUE obj)
 
     if (MATRIX_P(argv[0])) {
       return rb_gsl_wavelet2d(argc, argv, obj,
-			      gsl_wavelet2d_transform_matrix);
+			      gsl_wavelet2d_transform_matrix, sss);
     }
     if (VECTOR_P(obj)) {
       CHECK_WAVELET(argv[0]);
       Data_Get_Struct(argv[0], gsl_wavelet, w);
       Data_Get_Struct(obj, gsl_vector, v);
-    } else {
-      CHECK_VECTOR(argv[0]);
+      ret = obj;
+      ptr1 = v->data;
+      n = v->size;
+      stride = v->stride;
+    } else if (VECTOR_P(argv[0])) {
+
+      CHECK_WAVELET(obj);
       Data_Get_Struct(obj, gsl_wavelet, w);
       Data_Get_Struct(argv[0], gsl_vector, v);
+      ret = argv[0];
+      ptr1 = v->data;
+      n = v->size;
+      stride = v->stride;
+#ifdef HAVE_NARRAY_H
+    } else if (NA_IsNArray(obj)) {
+      CHECK_WAVELET(argv[0]);
+      Data_Get_Struct(argv[0], gsl_wavelet, w);
+      GetNArray(obj, na1);
+      ret = obj;
+      ptr1 = (double*) na1->ptr;
+      n = na1->total;
+      naflag = 1;
+      stride = 1;
+    } else if (NA_IsNArray(argv[0])) {
+      CHECK_WAVELET(obj);
+      Data_Get_Struct(obj, gsl_wavelet, w);
+      GetNArray(argv[0], na1);
+      ret = argv[0];
+      ptr1 = (double*) na1->ptr;
+      n = na1->total;
+      naflag = 1;
+      stride = 1;
+#endif
+    } else {
+      rb_raise(rb_eTypeError, "wrong argument type");
     }
     itmp = 1;
     break;
@@ -244,24 +310,59 @@ static VALUE rb_gsl_wavelet_transform(int argc, VALUE *argv, VALUE obj)
     rb_raise(rb_eArgError, "too many arguments");
     break;
   }
-  status = gsl_wavelet_transform(w, v->data, v->stride, v->size, dir, work);
+  if (naflag == 0) {
+    if (sss == RB_GSL_DWT_COPY) { 
+      vnew = gsl_vector_alloc(v->size);
+      gsl_vector_memcpy(vnew, v);
+      ary = Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, vnew);
+      ptr2 = vnew->data;
+    } else {
+      ary = ret;
+      ptr2 = ptr1;
+    }
+  } else {
+#ifdef HAVE_NARRAY_H
+    if (sss == RB_GSL_DWT_COPY) {
+      ary = na_make_object(NA_DFLOAT, na1->rank, na1->shape, cNArray);
+      ptr2 = NA_PTR_TYPE(ary, double*);
+      memcpy(ptr2, ptr1, sizeof(double)*n);
+    } else {
+      ary = ret;
+      ptr2 = ptr1;
+    }
+#endif
+  }
+  status = gsl_wavelet_transform(w, ptr2, stride, n, dir, work);
   if (flag) gsl_wavelet_workspace_free(work);
-  return INT2FIX(status);
+  return ary;
+}
+
+static VALUE rb_gsl_wavelet_transform(int argc, VALUE *argv, VALUE obj)
+{
+  return rb_gsl_wavelet_transform0(argc, argv, obj, RB_GSL_DWT_COPY);
+}
+
+static VALUE rb_gsl_wavelet_transform2(int argc, VALUE *argv, VALUE obj)
+{
+  return rb_gsl_wavelet_transform0(argc, argv, obj, RB_GSL_DWT_INPLACE);
 }
 
 static VALUE rb_gsl_wavelet_trans(int argc, VALUE *argv, VALUE obj,
 				  int (*trans)(const gsl_wavelet *, 
 					       double *, size_t, size_t, 
-					       gsl_wavelet_workspace *));
-static VALUE rb_gsl_wavelet_trans(int argc, VALUE *argv, VALUE obj,
-				  int (*trans)(const gsl_wavelet *, 
-					       double *, size_t, size_t, 
-					       gsl_wavelet_workspace *))
+					       gsl_wavelet_workspace *),
+				  int sss)
 {
   gsl_wavelet *w = NULL;
-  gsl_vector *v = NULL;
+  gsl_vector *v = NULL, *vnew;
   gsl_wavelet_workspace *work = NULL;
-  int itmp, flag = 0, status;
+  int itmp, flag = 0, status, naflag = 0;
+  double *ptr1 = NULL, *ptr2 = NULL;
+  size_t n, stride;
+  VALUE ary, ret;
+#ifdef HAVE_NARRAY_H
+  struct NARRAY *na;
+#endif
   switch (TYPE(obj)) {
   case T_MODULE:
   case T_CLASS:
@@ -272,16 +373,31 @@ static VALUE rb_gsl_wavelet_trans(int argc, VALUE *argv, VALUE obj,
     if (MATRIX_P(argv[1])) {
       if (trans == gsl_wavelet_transform_forward) {
 	return rb_gsl_wavelet2d_trans(argc, argv, obj, 
-				      gsl_wavelet2d_transform_matrix_forward);
+				      gsl_wavelet2d_transform_matrix_forward, sss);
       } else {
 	return rb_gsl_wavelet2d_trans(argc, argv, obj, 
-				      gsl_wavelet2d_transform_matrix_inverse);
+				      gsl_wavelet2d_transform_matrix_inverse, sss);
       }
     }
-
-    CHECK_VECTOR(argv[1]);
-    Data_Get_Struct(argv[0], gsl_wavelet, w);
-    Data_Get_Struct(argv[1], gsl_vector, v);
+    if (VECTOR_P(argv[1])) {
+      Data_Get_Struct(argv[0], gsl_wavelet, w);
+      Data_Get_Struct(argv[1], gsl_vector, v);
+      ret = argv[1];
+      ptr1 = v->data;
+      n = v->size;
+      stride = v->stride;
+#ifdef HAVE_NARRAY_H
+    } else if (NA_IsNArray(argv[1])) {
+      GetNArray(argv[1], na);
+      ret = argv[1];
+      ptr1 = (double*) na->ptr;
+      n = na->total;
+      naflag = 1;
+      stride = 1;
+#endif
+    } else {
+      rb_raise(rb_eTypeError, "wrong argument type (Vector expected)");
+    }
     itmp = 2;
     break;
   default:
@@ -290,20 +406,50 @@ static VALUE rb_gsl_wavelet_trans(int argc, VALUE *argv, VALUE obj,
     if (MATRIX_P(argv[0])) {
       if (trans == gsl_wavelet_transform_forward) {
 	return rb_gsl_wavelet2d_trans(argc, argv, obj, 
-				      gsl_wavelet2d_transform_matrix_forward);
+				      gsl_wavelet2d_transform_matrix_forward, sss);
       } else {
 	return rb_gsl_wavelet2d_trans(argc, argv, obj, 
-				      gsl_wavelet2d_transform_matrix_inverse);
+				      gsl_wavelet2d_transform_matrix_inverse, sss);
       }
     }
     if (VECTOR_P(obj)) {
       CHECK_WAVELET(argv[0]);
       Data_Get_Struct(argv[0], gsl_wavelet, w);
       Data_Get_Struct(obj, gsl_vector, v);
-    } else {
-      CHECK_VECTOR(argv[0]);
+      ret = obj;
+      ptr1 = v->data;
+      n = v->size;
+      stride = v->stride;
+    } else if (VECTOR_P(argv[0])) {
+      CHECK_WAVELET(obj);
       Data_Get_Struct(obj, gsl_wavelet, w);
       Data_Get_Struct(argv[0], gsl_vector, v);
+      ret = argv[0];
+      ptr1 = v->data;
+      n = v->size;
+      stride = v->stride;
+#ifdef HAVE_NARRAY_H
+    } else if (NA_IsNArray(obj)) {
+      CHECK_WAVELET(argv[0]);
+      Data_Get_Struct(argv[0], gsl_wavelet, w);
+      GetNArray(obj, na);
+      ret = obj;
+      ptr1 = (double*) na->ptr;
+      n = na->total;
+      naflag = 1;
+      stride = 1;
+    } else if (NA_IsNArray(argv[0])) {
+      CHECK_WAVELET(obj);
+      Data_Get_Struct(obj, gsl_wavelet, w);
+      GetNArray(argv[0], na);
+      ret = argv[0];
+      ptr1 = (double*) na->ptr;
+      n = na->total;
+      naflag = 1;
+      stride = 1;
+#endif
+    } else {
+      rb_raise(rb_eTypeError, "wrong argument type");
     }
     itmp = 1;
     break;
@@ -321,31 +467,69 @@ static VALUE rb_gsl_wavelet_trans(int argc, VALUE *argv, VALUE obj,
     rb_raise(rb_eArgError, "too many arguments");
     break;
   }
-  status = (*trans)(w, v->data, v->stride, v->size, work);
+  if (naflag == 0) {
+    if (sss == RB_GSL_DWT_COPY) {
+      vnew = gsl_vector_alloc(v->size);
+      gsl_vector_memcpy(vnew, v);
+      ary = Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, vnew);
+      ptr2 = vnew->data;
+    } else {
+      ptr2 = ptr1;
+      ary = ret;
+    }
+  } else {
+#ifdef HAVA_NARRAY_H
+    if (sss == RB_GSL_DWT_COPY) {
+      ary = na_make_object(NA_DFLOAT, na->rank, na->shape, cNArray);
+      ptr2 = NA_PTR_TYPE(ary, double*);
+      memcpy(ptr2, ptr1, sizeof(double)*n);
+    } else {
+      ptr2 = ptr1;
+      ary = ret;
+    }
+#endif
+  }
+  status = (*trans)(w, ptr2, stride, n, work);
   if (flag) gsl_wavelet_workspace_free(work);
-  return INT2FIX(status);
+  return ary;
 }
 
 static VALUE rb_gsl_wavelet_transform_forward(int argc, VALUE *argv, VALUE obj)
 {
-  return rb_gsl_wavelet_trans(argc, argv, obj, gsl_wavelet_transform_forward);
+  return rb_gsl_wavelet_trans(argc, argv, obj, gsl_wavelet_transform_forward,
+			      RB_GSL_DWT_COPY);
 }
 
 static VALUE rb_gsl_wavelet_transform_inverse(int argc, VALUE *argv, VALUE obj)
 {
-  return rb_gsl_wavelet_trans(argc, argv, obj, gsl_wavelet_transform_inverse);
+  return rb_gsl_wavelet_trans(argc, argv, obj, gsl_wavelet_transform_inverse,
+			      RB_GSL_DWT_COPY);
+}
+
+static VALUE rb_gsl_wavelet_transform_forward2(int argc, VALUE *argv, VALUE obj)
+{
+  return rb_gsl_wavelet_trans(argc, argv, obj, gsl_wavelet_transform_forward,
+			      RB_GSL_DWT_INPLACE);
+}
+
+static VALUE rb_gsl_wavelet_transform_inverse2(int argc, VALUE *argv, VALUE obj)
+{
+  return rb_gsl_wavelet_trans(argc, argv, obj, gsl_wavelet_transform_inverse,
+			      RB_GSL_DWT_INPLACE);
 }
 
 static VALUE rb_gsl_wavelet2d(int argc, VALUE *argv, VALUE obj,
 			      int (*trans)(const gsl_wavelet *, 
 					   gsl_matrix *, 
 					   gsl_wavelet_direction, 
-					   gsl_wavelet_workspace *))
+					   gsl_wavelet_workspace *),
+			      int sss)
 {
   gsl_wavelet *w = NULL;
-  gsl_matrix *m = NULL;
+  gsl_matrix *m = NULL, *mnew;
   gsl_wavelet_direction dir = forward;
   gsl_wavelet_workspace *work = NULL;
+  VALUE ary, ret;
   int itmp, flag = 0, status;
   switch (TYPE(obj)) {
   case T_MODULE:
@@ -354,6 +538,7 @@ static VALUE rb_gsl_wavelet2d(int argc, VALUE *argv, VALUE obj,
     if (argc < 2) rb_raise(rb_eArgError, "too few arguments");
     CHECK_WAVELET(argv[0]);
     CHECK_MATRIX(argv[1]);
+    ret = argv[1];
     Data_Get_Struct(argv[0], gsl_wavelet, w);
     Data_Get_Struct(argv[1], gsl_matrix, m);
     itmp = 2;
@@ -364,10 +549,12 @@ static VALUE rb_gsl_wavelet2d(int argc, VALUE *argv, VALUE obj,
       CHECK_WAVELET(argv[0]);
       Data_Get_Struct(argv[0], gsl_wavelet, w);
       Data_Get_Struct(obj, gsl_matrix, m);
+      ret = obj;
     } else {
       CHECK_MATRIX(argv[0]);
       Data_Get_Struct(obj, gsl_wavelet, w);
       Data_Get_Struct(argv[0], gsl_matrix, m);
+      ret = argv[0];
     }
     itmp = 1;
     break;
@@ -398,24 +585,40 @@ static VALUE rb_gsl_wavelet2d(int argc, VALUE *argv, VALUE obj,
     rb_raise(rb_eArgError, "too many arguments");
     break;
   }
-  status = (*trans)(w, m, dir, work);
+  if (sss == RB_GSL_DWT_COPY) {
+    mnew = make_matrix_clone(m);
+    ary = Data_Wrap_Struct(cgsl_matrix, 0, gsl_matrix_free, mnew);
+  } else {
+    mnew = m;
+    ary = ret;
+  }
+  status = (*trans)(w, mnew, dir, work);
   if (flag) gsl_wavelet_workspace_free(work);
-  return INT2FIX(status);
+  return ary;
 }
 
 static VALUE rb_gsl_wavelet2d_transform_matrix(int argc, VALUE *argv, VALUE obj)
 {
-  return rb_gsl_wavelet2d(argc, argv, obj, gsl_wavelet2d_transform_matrix);
+  return rb_gsl_wavelet2d(argc, argv, obj, gsl_wavelet2d_transform_matrix,
+			  RB_GSL_DWT_COPY);
+}
+
+static VALUE rb_gsl_wavelet2d_transform_matrix2(int argc, VALUE *argv, VALUE obj)
+{
+  return rb_gsl_wavelet2d(argc, argv, obj, gsl_wavelet2d_transform_matrix,
+			  RB_GSL_DWT_INPLACE);
 }
 
 static VALUE rb_gsl_wavelet2d_trans(int argc, VALUE *argv, VALUE obj,
 				    int (*trans)(const gsl_wavelet *, 
 						 gsl_matrix *,
-						 gsl_wavelet_workspace *))
+						 gsl_wavelet_workspace *),
+				    int sss)
 {
   gsl_wavelet *w = NULL;
-  gsl_matrix *m = NULL;
+  gsl_matrix *m = NULL, *mnew;
   gsl_wavelet_workspace *work = NULL;
+  VALUE ary, ret;
   int itmp, flag = 0, status;
   switch (TYPE(obj)) {
   case T_MODULE:
@@ -426,6 +629,7 @@ static VALUE rb_gsl_wavelet2d_trans(int argc, VALUE *argv, VALUE obj,
     CHECK_MATRIX(argv[1]);
     Data_Get_Struct(argv[0], gsl_wavelet, w);
     Data_Get_Struct(argv[1], gsl_matrix, m);
+    ret = argv[1];
     itmp = 2;
     break;
   default:
@@ -434,10 +638,12 @@ static VALUE rb_gsl_wavelet2d_trans(int argc, VALUE *argv, VALUE obj,
       CHECK_WAVELET(argv[0]);
       Data_Get_Struct(argv[0], gsl_wavelet, w);
       Data_Get_Struct(obj, gsl_matrix, m);
+      ret = obj;
     } else {
       CHECK_MATRIX(argv[0]);
       Data_Get_Struct(obj, gsl_wavelet, w);
       Data_Get_Struct(argv[0], gsl_matrix, m);
+      ret = argv[0];
     }
     itmp = 1;
     break;
@@ -455,39 +661,85 @@ static VALUE rb_gsl_wavelet2d_trans(int argc, VALUE *argv, VALUE obj,
     rb_raise(rb_eArgError, "too many arguments");
     break;
   }
-  status = (*trans)(w, m, work);
+  if (sss == RB_GSL_DWT_COPY) {
+    mnew = make_matrix_clone(m);
+    ary = Data_Wrap_Struct(cgsl_matrix, 0, gsl_matrix_free, mnew);
+  } else {
+    mnew = m;
+    ary = ret;
+  }
+  status = (*trans)(w, mnew, work);
   if (flag) gsl_wavelet_workspace_free(work);
-  return INT2FIX(status);
+  return ary;
 }
 
 static VALUE rb_gsl_wavelet2d_transform_matrix_forward(int argc, VALUE *argv, VALUE obj)
 {
   return rb_gsl_wavelet2d_trans(argc, argv, obj, 
-				gsl_wavelet2d_transform_matrix_forward);
+				gsl_wavelet2d_transform_matrix_forward,
+				RB_GSL_DWT_COPY);
+}
+
+static VALUE rb_gsl_wavelet2d_transform_matrix_forward2(int argc, VALUE *argv, VALUE obj)
+{
+  return rb_gsl_wavelet2d_trans(argc, argv, obj, 
+				gsl_wavelet2d_transform_matrix_forward,
+				RB_GSL_DWT_INPLACE);
 }
 
 static VALUE rb_gsl_wavelet2d_transform_matrix_inverse(int argc, VALUE *argv, VALUE obj)
 {
   return rb_gsl_wavelet2d_trans(argc, argv, obj, 
-				gsl_wavelet2d_transform_matrix_inverse);
+				gsl_wavelet2d_transform_matrix_inverse,
+				RB_GSL_DWT_COPY);
+}
+
+static VALUE rb_gsl_wavelet2d_transform_matrix_inverse2(int argc, VALUE *argv, VALUE obj)
+{
+  return rb_gsl_wavelet2d_trans(argc, argv, obj, 
+				gsl_wavelet2d_transform_matrix_inverse,
+				RB_GSL_DWT_INPLACE);
 }
 
 /** nstransform **/
 static VALUE rb_gsl_wavelet2d_nstransform_matrix(int argc, VALUE *argv, VALUE obj)
 {
-  return rb_gsl_wavelet2d(argc, argv, obj, gsl_wavelet2d_nstransform_matrix);
+  return rb_gsl_wavelet2d(argc, argv, obj, gsl_wavelet2d_nstransform_matrix,
+			  RB_GSL_DWT_COPY);
+}
+
+static VALUE rb_gsl_wavelet2d_nstransform_matrix2(int argc, VALUE *argv, VALUE obj)
+{
+  return rb_gsl_wavelet2d(argc, argv, obj, gsl_wavelet2d_nstransform_matrix,
+			  RB_GSL_DWT_INPLACE);
 }
 
 static VALUE rb_gsl_wavelet2d_nstransform_matrix_forward(int argc, VALUE *argv, VALUE obj)
 {
   return rb_gsl_wavelet2d_trans(argc, argv, obj, 
-				gsl_wavelet2d_nstransform_matrix_forward);
+				gsl_wavelet2d_nstransform_matrix_forward,
+				RB_GSL_DWT_COPY);
+}
+
+static VALUE rb_gsl_wavelet2d_nstransform_matrix_forward2(int argc, VALUE *argv, VALUE obj)
+{
+  return rb_gsl_wavelet2d_trans(argc, argv, obj, 
+				gsl_wavelet2d_nstransform_matrix_forward,
+				RB_GSL_DWT_INPLACE);
 }
 
 static VALUE rb_gsl_wavelet2d_nstransform_matrix_inverse(int argc, VALUE *argv, VALUE obj)
 {
   return rb_gsl_wavelet2d_trans(argc, argv, obj, 
-				gsl_wavelet2d_nstransform_matrix_inverse);
+				gsl_wavelet2d_nstransform_matrix_inverse,
+				RB_GSL_DWT_COPY);
+}
+
+static VALUE rb_gsl_wavelet2d_nstransform_matrix_inverse2(int argc, VALUE *argv, VALUE obj)
+{
+  return rb_gsl_wavelet2d_trans(argc, argv, obj, 
+				gsl_wavelet2d_nstransform_matrix_inverse,
+				RB_GSL_DWT_INPLACE);
 }
 
 #endif
@@ -496,7 +748,7 @@ void Init_wavelet(VALUE module)
 {
   VALUE cgsl_wavelet2d;
 
-  cgsl_wavelet = rb_define_class_under(module, "Wavelet", rb_cObject);
+  cgsl_wavelet = rb_define_class_under(module, "Wavelet", cGSL_Object);
   cgsl_wavelet2d = rb_define_class_under(module, "Wavelet2d", cgsl_wavelet);
 
   rb_define_singleton_method(cgsl_wavelet, "new", rb_gsl_wavelet_new, 2);
@@ -506,7 +758,7 @@ void Init_wavelet(VALUE module)
   rb_gsl_wavelet_define_const(cgsl_wavelet);
   rb_define_method(cgsl_wavelet, "name", rb_gsl_wavelet_name, 0);
 
-  cgsl_wavelet_workspace = rb_define_class_under(cgsl_wavelet, "Workspace", rb_cObject);
+  cgsl_wavelet_workspace = rb_define_class_under(cgsl_wavelet, "Workspace", cGSL_Object);
   rb_define_singleton_method(cgsl_wavelet_workspace, "new", 
 			     rb_gsl_wavelet_workspace_new, 1);
   rb_define_singleton_method(cgsl_wavelet_workspace, "alloc", 
@@ -518,6 +770,12 @@ void Init_wavelet(VALUE module)
 			     rb_gsl_wavelet_transform, -1);
   rb_define_method(cgsl_wavelet, "transform", rb_gsl_wavelet_transform, -1);
   rb_define_method(cgsl_vector, "wavelet_transform", rb_gsl_wavelet_transform, -1);
+  rb_define_singleton_method(cgsl_wavelet, "transform!", 
+			     rb_gsl_wavelet_transform2, -1);
+  rb_define_method(cgsl_wavelet, "transform!", rb_gsl_wavelet_transform2, -1);
+  rb_define_method(cgsl_vector, "wavelet_transform!", rb_gsl_wavelet_transform2, -1);
+
+  /**/
 
   rb_define_singleton_method(cgsl_wavelet, "transform_forward", 
 			     rb_gsl_wavelet_transform_forward, -1);
@@ -534,6 +792,22 @@ void Init_wavelet(VALUE module)
   rb_define_alias(cgsl_wavelet, "inverse", "transform_inverse");
   rb_define_method(cgsl_vector, "wavelet_transform_inverse", 
 		   rb_gsl_wavelet_transform_inverse, -1);
+
+  rb_define_singleton_method(cgsl_wavelet, "transform_forward!", 
+			     rb_gsl_wavelet_transform_forward2, -1);
+  rb_define_method(cgsl_wavelet, "transform_forward!", 
+		   rb_gsl_wavelet_transform_forward2, -1);
+  rb_define_alias(cgsl_wavelet, "forward!", "transform_forward!");
+  rb_define_method(cgsl_vector, "wavelet_transform_forward!", 
+		   rb_gsl_wavelet_transform_forward2, -1);
+
+  rb_define_singleton_method(cgsl_wavelet, "transform_inverse!", 
+			     rb_gsl_wavelet_transform_inverse2, -1);
+  rb_define_method(cgsl_wavelet, "transform_inverse!", 
+		   rb_gsl_wavelet_transform_inverse2, -1);
+  rb_define_alias(cgsl_wavelet, "inverse!", "transform_inverse!");
+  rb_define_method(cgsl_vector, "wavelet_transform_inverse!", 
+		   rb_gsl_wavelet_transform_inverse2, -1);
   /***** 2d *****/
   rb_define_singleton_method(cgsl_wavelet, "transform_matrix", 
 			     rb_gsl_wavelet2d_transform_matrix, -1);
@@ -546,6 +820,18 @@ void Init_wavelet(VALUE module)
   rb_define_method(cgsl_matrix, "wavelet_transform", 
 		   rb_gsl_wavelet2d_transform_matrix, -1);
 
+  rb_define_singleton_method(cgsl_wavelet, "transform_matrix!", 
+			     rb_gsl_wavelet2d_transform_matrix2, -1);
+  rb_define_singleton_method(cgsl_wavelet2d, "transform!", 
+			     rb_gsl_wavelet2d_transform_matrix2, -1);
+  rb_define_method(cgsl_wavelet, "transform_matrix!",
+		   rb_gsl_wavelet2d_transform_matrix2, -1);
+  rb_define_method(cgsl_wavelet2d, "transform!", 
+		   rb_gsl_wavelet2d_transform_matrix2, -1);
+  rb_define_method(cgsl_matrix, "wavelet_transform!", 
+		   rb_gsl_wavelet2d_transform_matrix2, -1);
+  /**/
+
   rb_define_singleton_method(cgsl_wavelet, "transform_matrix_forward", 
 			     rb_gsl_wavelet2d_transform_matrix_forward, -1);
   rb_define_singleton_method(cgsl_wavelet2d, "transform_forward", 
@@ -557,6 +843,17 @@ void Init_wavelet(VALUE module)
   rb_define_method(cgsl_matrix, "wavelet_transform_forward", 
 		   rb_gsl_wavelet2d_transform_matrix_forward, -1);
 
+  rb_define_singleton_method(cgsl_wavelet, "transform_matrix_forward!", 
+			     rb_gsl_wavelet2d_transform_matrix_forward2, -1);
+  rb_define_singleton_method(cgsl_wavelet2d, "transform_forward!", 
+			     rb_gsl_wavelet2d_transform_matrix_forward2, -1);
+  rb_define_method(cgsl_wavelet, "transform_matrix_forward!", 
+		   rb_gsl_wavelet2d_transform_matrix_forward2, -1);
+  rb_define_method(cgsl_wavelet2d, "transform_forward!", 
+		   rb_gsl_wavelet2d_transform_matrix_forward2, -1);
+  rb_define_method(cgsl_matrix, "wavelet_transform_forward!", 
+		   rb_gsl_wavelet2d_transform_matrix_forward2, -1);
+
   rb_define_singleton_method(cgsl_wavelet, "transform_matrix_inverse", 
 			     rb_gsl_wavelet2d_transform_matrix_inverse, -1);
   rb_define_singleton_method(cgsl_wavelet2d, "transform_inverse", 
@@ -568,6 +865,17 @@ void Init_wavelet(VALUE module)
   rb_define_method(cgsl_matrix, "wavelet_transform_inverse", 
 		   rb_gsl_wavelet2d_transform_matrix_inverse, -1);
 
+  rb_define_singleton_method(cgsl_wavelet, "transform_matrix_inverse!", 
+			     rb_gsl_wavelet2d_transform_matrix_inverse2, -1);
+  rb_define_singleton_method(cgsl_wavelet2d, "transform_inverse!", 
+			     rb_gsl_wavelet2d_transform_matrix_inverse2, -1);
+  rb_define_method(cgsl_wavelet, "transform_matrix_inverse!", 
+		   rb_gsl_wavelet2d_transform_matrix_inverse2, -1);
+  rb_define_method(cgsl_wavelet2d, "transform_inverse!", 
+		   rb_gsl_wavelet2d_transform_matrix_inverse2, -1);
+  rb_define_method(cgsl_matrix, "wavelet_transform_inverse!", 
+		   rb_gsl_wavelet2d_transform_matrix_inverse2, -1);
+
   /** nstransform **/
   rb_define_singleton_method(cgsl_wavelet, "nstransform_matrix", 
 			     rb_gsl_wavelet2d_nstransform_matrix, -1);
@@ -578,6 +886,16 @@ void Init_wavelet(VALUE module)
   rb_define_method(cgsl_wavelet2d, "nstransform", 
 		   rb_gsl_wavelet2d_nstransform_matrix, -1);
 
+  rb_define_singleton_method(cgsl_wavelet, "nstransform_matrix!", 
+			     rb_gsl_wavelet2d_nstransform_matrix2, -1);
+  rb_define_singleton_method(cgsl_wavelet2d, "nstransform!", 
+			     rb_gsl_wavelet2d_nstransform_matrix2, -1);
+  rb_define_method(cgsl_wavelet, "nstransform_matrix!", 
+		   rb_gsl_wavelet2d_nstransform_matrix2, -1);
+  rb_define_method(cgsl_wavelet2d, "nstransform!", 
+		   rb_gsl_wavelet2d_nstransform_matrix2, -1);
+  /**/
+
   rb_define_singleton_method(cgsl_wavelet, "nstransform_matrix_forward", 
 			     rb_gsl_wavelet2d_nstransform_matrix_forward, -1);
   rb_define_singleton_method(cgsl_wavelet2d, "nstransform_forward", 
@@ -587,6 +905,15 @@ void Init_wavelet(VALUE module)
   rb_define_method(cgsl_wavelet2d, "nstransform_forward", 
 		   rb_gsl_wavelet2d_nstransform_matrix_forward, -1);
 
+  rb_define_singleton_method(cgsl_wavelet, "nstransform_matrix_forward!", 
+			     rb_gsl_wavelet2d_nstransform_matrix_forward2, -1);
+  rb_define_singleton_method(cgsl_wavelet2d, "nstransform_forward!", 
+			     rb_gsl_wavelet2d_nstransform_matrix_forward2, -1);
+  rb_define_method(cgsl_wavelet, "nstransform_matrix_forward!", 
+		   rb_gsl_wavelet2d_nstransform_matrix_forward2, -1);
+  rb_define_method(cgsl_wavelet2d, "nstransform_forward!", 
+		   rb_gsl_wavelet2d_nstransform_matrix_forward2, -1);
+
   rb_define_singleton_method(cgsl_wavelet, "nstransform_matrix_inverse", 
 			     rb_gsl_wavelet2d_nstransform_matrix_inverse, -1);
   rb_define_singleton_method(cgsl_wavelet2d, "nstransform_inverse", 
@@ -596,10 +923,18 @@ void Init_wavelet(VALUE module)
   rb_define_method(cgsl_wavelet2d, "nstransform_inverse", 
 		   rb_gsl_wavelet2d_nstransform_matrix_inverse, -1);
 
+  rb_define_singleton_method(cgsl_wavelet, "nstransform_matrix_inverse!", 
+			     rb_gsl_wavelet2d_nstransform_matrix_inverse2, -1);
+  rb_define_singleton_method(cgsl_wavelet2d, "nstransform_inverse!", 
+			     rb_gsl_wavelet2d_nstransform_matrix_inverse2, -1);
+  rb_define_method(cgsl_wavelet, "nstransform_matrix_inverse!", 
+		   rb_gsl_wavelet2d_nstransform_matrix_inverse2, -1);
+  rb_define_method(cgsl_wavelet2d, "nstransform_inverse!", 
+		   rb_gsl_wavelet2d_nstransform_matrix_inverse2, -1);
+
 #endif
 }
 
 #undef CHECK_WAVELET
 #undef CHECK_WORKSPACE
 
-
