@@ -8,6 +8,6 @@ puts("Derivative: -exp(-x)")
 
 h = 1e-8
 x = Vector.linspace(0, 5, 50)
-y, err = f.eval(x)
+y = f.eval(x)
 dy, derr = f.deriv_central(x, h)
 graph(x, y, dy, "-T X -C -g 3 -X x -L 'f(x) = exp(-x), and its derivative'")
