@@ -1,8 +1,8 @@
 #!/usr/bin/env ruby
 require("gsl")
 
-m = Matrix.new([1.0, 1/2.0, 1/3.0, 1/4.0], [1/2.0, 1/3.0, 1/4.0, 1/5.0],
-		   [1/3.0, 1/4.0, 1/5.0, 1/6.0], [1/4.0, 1/5.0, 1/6.0, 1/7.0])
+m = GSL::Matrix[[1.0, 1/2.0, 1/3.0, 1/4.0], [1/2.0, 1/3.0, 1/4.0, 1/5.0],
+           [1/3.0, 1/4.0, 1/5.0, 1/6.0], [1/4.0, 1/5.0, 1/6.0, 1/7.0]]
 
 eigval = m.eigen_symm
 p eigval
@@ -11,6 +11,8 @@ eigval, eigvec = m.eigen_symmv
 
 p eigval == Eigen.symm(m)
 val, vec = Eigen.symmv(m)
+p vec
+
 i = 0
 vec.each_col do |v|
   a = (m*v)/v
