@@ -7,11 +7,16 @@ rng = Rng.alloc
 
 h = Histogram.new(20, [0, 20])
 
+=begin
 for i in 0...N do
   r = rng.poisson(5)
 #  r = Ran::poisson(rng, 5)
   h.increment(r)
 end
+=end
+
+v = rng.poisson(5, N)
+h.fill(v)
 
 h.normalize!
 x = Vector.linspace(0, 20, 100)
@@ -19,3 +24,4 @@ y = Ran::poisson_pdf(x, 5)
 graph(h, [x, y], "-C -g 3 -L 'Poisson distribution, mu = 5'")
 
 
+
