@@ -1,8 +1,8 @@
 #!/usr/bin/env ruby
 require("gsl")
 
-m = Matrix::new([0.18, 0.60, 0.57, 0.96], [0.41, 0.24, 0.99, 0.58],
-		    [0.14, 0.30, 0.97, 0.66], [0.51, 0.13, 0.19, 0.85])
+m = Matrix[[0.18, 0.60, 0.57, 0.96], [0.41, 0.24, 0.99, 0.58],
+           [0.14, 0.30, 0.97, 0.66], [0.51, 0.13, 0.19, 0.85]]
 qr, tau = m.QR_decomp
 
 b = Vector[1, 2, 3, 4]
