@@ -3,9 +3,7 @@ require("gsl")
 
 N = 10000
 r = GSL::Rng.new
-h = Histogram.new(100, [-4, 4])
-N.times {
-   h.increment(r.gaussian)
-}
+v = r.gaussian(1, N)
+h = v.histogram(100, -4, 4)
 h.graph
 
