@@ -3,9 +3,7 @@ require("gsl")
 
 N = 10000
 r = GSL::Rng.new
-h = Histogram.new(20, [0, 20])
-N.times {
-   h.increment(r.poisson(5))
-}
+v = r.poisson(5, N)
+h = v.histogram(20, 0, 20)
 h.graph
 
